@@ -1,21 +1,24 @@
 /*
- * Copyright (c) 2001 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2001-2002 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -35,9 +38,12 @@
 #define CJK_KATAKANA	(CJK_JAPAN | CJK_CHINESE_SIMP | CJK_KOREAN)
 
 
-/* Remeber the las unique CJK bit */
+/* Remember the last unique CJK bit */
 u_int8_t cjk_lastunique = 0;
 
+/* CJK encoding bias */
+u_int32_t hfs_encodingbias = 0;
+
 
 /* Map CJK bits to Mac encoding */
 u_int8_t cjk_encoding[] = {
@@ -808,7 +814,33 @@ hfs_pickencoding(const u_int16_t *src, int len)
 	if (cjkstate) {
 		if (powerof2(cjkstate)) {
 			cjk_lastunique = cjkstate;
-		} else if (cjk_lastunique) {
+			return ((u_int32_t)cjk_encoding[cjkstate]);
+		} 
+		if (hfs_encodingbias != 0) {
+			switch(hfs_encodingbias) {
+			case kTextEncodingMacJapanese:
+				if (cjkstate & CJK_JAPAN)
+					return (kTextEncodingMacJapanese);
+				break;
+			case kTextEncodingMacKorean:
+				if (cjkstate & CJK_KOREAN)
+					return (kTextEncodingMacKorean);
+				break;
+			case kTextEncodingMacChineseTrad:
+				if (cjkstate & CJK_CHINESE_TRAD)
+					return (kTextEncodingMacChineseTrad);
+				if (cjkstate & CJK_CHINESE_SIMP)
+					return (kTextEncodingMacChineseSimp);
+				break;
+			case kTextEncodingMacChineseSimp:
+				if (cjkstate & CJK_CHINESE_SIMP)
+					return (kTextEncodingMacChineseSimp);
+				if (cjkstate & CJK_CHINESE_TRAD)
+					return (kTextEncodingMacChineseTrad);
+				break;
+			}
+		}
+		if (cjk_lastunique) {
 			if (cjkstate & cjk_lastunique)
 				cjkstate = cjk_lastunique;
 			else
