@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -50,9 +53,6 @@ hfs_swap_HFSPlusForkData (
 {
     int i;
 
-    DBG_FUNC_NAME("hfs_swap_HFSPlusForkData");
-    DBG_PRINT_FUNC_NAME();
-
 	src->logicalSize		= SWAP_BE64 (src->logicalSize);
 
 	src->clumpSize			= SWAP_BE32 (src->clumpSize);
@@ -84,8 +84,6 @@ hfs_swap_BTNode (
     UInt32 i;
     int error = 0;
 
-    DBG_FUNC_NAME("hfs_swap_BTNode");
-    DBG_PRINT_FUNC_NAME();
 
 #ifdef ENDIAN_DEBUG
     if (unswap == 0) {
@@ -218,9 +216,6 @@ hfs_swap_HFSPlusBTInternalNode (
     UInt32 i;
     UInt32 j;
 
-    DBG_FUNC_NAME("hfs_swap_HFSPlusBTInternalNode");
-    DBG_PRINT_FUNC_NAME();
-
     if (fileID == kHFSExtentsFileID) {
         HFSPlusExtentKey *srcKey;
         HFSPlusExtentDescriptor *srcRec;
@@ -383,9 +378,6 @@ hfs_swap_HFSBTInternalNode (
     UInt32 i;
     UInt32 j;
 
-    DBG_FUNC_NAME("hfs_swap_HFSBTInternalNode");
-    DBG_PRINT_FUNC_NAME();
-
     if (fileID == kHFSExtentsFileID) {
         HFSExtentKey *srcKey;
         HFSExtentDescriptor *srcRec;
