@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -71,6 +74,11 @@
 #ifndef	__BTREESINTERNAL__
 #define __BTREESINTERNAL__
 
+#include <sys/appleapiopts.h>
+
+#ifdef KERNEL
+#ifdef __APPLE_API_PRIVATE
+
 #ifndef __FILEMGRINTERNAL__
 #include "FileMgrInternal.h"
 #endif
@@ -110,16 +118,17 @@ struct BlockDescriptor{
 	void		*blockHeader;
 	ByteCount	 blockSize;
 	Boolean		 blockReadFromDisk;
-	Byte		 reserved[3];
+	Byte         isModified;             // XXXdbg - for journaling
+	Byte		 reserved[2];
 };
 typedef struct BlockDescriptor BlockDescriptor;
 typedef BlockDescriptor *BlockDescPtr;
 
 
 struct FSBufferDescriptor {
-	LogicalAddress 					bufferAddress;
-	ByteCount 						itemSize;
-	ItemCount 						itemCount;
+	void *		bufferAddress;
+	ByteCount	itemSize;
+	ItemCount	itemCount;
 };
 typedef struct FSBufferDescriptor FSBufferDescriptor;
 
@@ -148,7 +157,7 @@ enum {
 };
 typedef OptionBits	ReleaseBlockOptions;
 
-typedef	UInt32	FSSize;
+typedef	UInt64	FSSize;
 typedef	UInt32	ForkBlockNumber;
 
 /*============================================================================
@@ -285,7 +294,6 @@ extern OSStatus	BTClosePath			(FCB		 				*filePtr );
 
 extern OSStatus	BTSearchRecord		(FCB		 				*filePtr,
 									 BTreeIterator				*searchIterator,
-									 UInt32						heuristicHint,
 									 FSBufferDescriptor			*btRecord,
 									 UInt16						*recordLen,
 									 BTreeIterator				*resultIterator );
@@ -310,6 +318,10 @@ extern OSStatus	BTReplaceRecord		(FCB		 				*filePtr,
 									 FSBufferDescriptor			*btRecord,
 									 UInt16						 recordLen );
 
+extern OSStatus	BTUpdateRecord		(FCB		 				*filePtr,
+									 BTreeIterator				*iterator,
+									 IterateCallBackProcPtr		 callBackProc,
+									 void						*callBackState );
 
 extern OSStatus	BTDeleteRecord		(FCB		 				*filePtr,
 									 BTreeIterator				*iterator );
@@ -330,4 +342,10 @@ extern OSStatus	BTGetLastSync		(FCB		 				*filePtr,
 extern OSStatus	BTSetLastSync		(FCB		 				*filePtr,
 									 UInt32						lastfsync );
 
+extern OSStatus	BTCheckFreeSpace	(FCB		 				*filePtr);
+
+extern OSStatus	BTHasContiguousNodes(FCB		 				*filePtr);
+
+#endif /* __APPLE_API_PRIVATE */
+#endif /* KERNEL */
 #endif // __BTREESINTERNAL__
