@@ -1,21 +1,24 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000, 2002 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -69,6 +72,11 @@
 #ifndef	__CATALOGPRIVATE__
 #define __CATALOGPRIVATE__
 
+#include <sys/appleapiopts.h>
+
+#ifdef KERNEL
+#ifdef __APPLE_API_PRIVATE
+
 #include "../../hfs_format.h"
 
 #include	"FileMgrInternal.h"
@@ -145,8 +153,6 @@ typedef struct CatalogCacheGlobals CatalogCacheGlobals;
 // Private Catalog Manager Routines (for use only by Catalog Manager, CatSearch and FileID Services)
 //
 
-extern	OSErr	LocateCatalogThread( const ExtendedVCB *volume, HFSCatalogNodeID nodeID, CatalogRecord *threadData,
-									 UInt16 *threadSize, UInt32 *threadHint);
 
 extern	OSErr	LocateCatalogNode(	const ExtendedVCB *volume, HFSCatalogNodeID folderID, const CatalogName *name,
 									UInt32 hint, CatalogKey *key, CatalogRecord *data, UInt32 *newHint);
@@ -160,19 +166,8 @@ extern OSErr	LocateCatalogRecord( const ExtendedVCB *volume, HFSCatalogNodeID fo
 extern OSErr	LocateCatalogNodeWithRetry ( const ExtendedVCB *volume, HFSCatalogNodeID folderID, ConstStr31Param pascalName,
 											 CatalogName *unicodeName, UInt32 hint, CatalogKey *keyPtr, CatalogRecord *dataPtr,
 											 UInt32 *newHint );
-
-extern OSErr	LocateCatalogNodeByMangledName( const ExtendedVCB *volume, HFSCatalogNodeID folderID, 
-				ConstStr31Param name, UInt32 length,
-				CatalogKey *keyPtr, CatalogRecord *dataPtr, UInt32 *hintPtr );
-
 extern OSErr	FlushCatalog( ExtendedVCB *volume);
 
-#define			InvalidateCatalogNodeCache(v, pid)
-
-extern OSErr	UpdateFolderCount( ExtendedVCB *volume, HFSCatalogNodeID parentID, const CatalogName *name, SInt16 newType,
-								   UInt32 hint, SInt16 valenceDelta);
-
-extern UInt16	GetCatalogRecordSize( const CatalogRecord *dataRecord);
 
 extern void		ConvertInputNameToUnicode(ConstStr31Param name, TextEncoding encodingHint,
 										  TextEncoding *actualEncoding, CatalogName *catalogName);
@@ -183,10 +178,6 @@ extern	void	BuildCatalogKey( HFSCatalogNodeID parentID, const CatalogName *name,
 extern	OSErr	BuildCatalogKeyUTF8(ExtendedVCB *volume, HFSCatalogNodeID parentID, const char *name,
 				    UInt32 length, CatalogKey *key, UInt32 *textEncoding);
 
-extern	void	UpdateCatalogName( ConstStr31Param srcName, Str31 destName);
-
-extern UInt32	CatalogNameLength( const CatalogName *name, Boolean isHFSPlus);
-
 extern void		CopyCatalogName( const CatalogName *srcName, CatalogName *dstName, Boolean isHFSPLus);
 
 extern OSErr	ResolveFileID( ExtendedVCB *vcb, HFSCatalogNodeID fileID, HFSCatalogNodeID *parentID, Str31 name );
@@ -197,16 +188,9 @@ extern OSErr	CreateFileThreadID( FIDParam *filePB, WDCBRecPtr *wdcbPtr );
 extern OSErr	ExchangeFiles( FIDParam *filePB, WDCBRecPtr *wdcbPtr );
 #endif 
 
-extern void		CopyCatalogNodeData( const ExtendedVCB *volume, const CatalogRecord *dataPtr, CatalogNodeData *nodeData);
-
-extern void		UpdateVolumeEncodings( ExtendedVCB *volume, TextEncoding encoding);
-
-extern void		AdjustVolumeCounts( ExtendedVCB *volume, SInt16 type, SInt16 delta );
-
 
 // Catalog Iterator Routines
 
-extern CatalogIterator* oGetCatalogIterator( const ExtendedVCB *volume, HFSCatalogNodeID folderID, UInt16 index);
 extern CatalogIterator* GetCatalogIterator(ExtendedVCB *volume, HFSCatalogNodeID folderID, UInt32 offset);
 
 extern OSErr	ReleaseCatalogIterator( CatalogIterator *catalogIterator );
@@ -220,4 +204,6 @@ extern void		UpdateBtreeIterator( const CatalogIterator *catalogIterator, BTreeI
 extern void		UpdateCatalogIterator( const BTreeIterator *btreeIterator, CatalogIterator *catalogIterator );
 
 
+#endif /* __APPLE_API_PRIVATE */
+#endif /* KERNEL */
 #endif //__CATALOGPRIVATE__
