@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -65,9 +68,10 @@ typedef struct {
 mythread_state_flavor_t thread_flavor_array[]={
 		{PPC_THREAD_STATE , PPC_THREAD_STATE_COUNT},
 		{PPC_FLOAT_STATE, PPC_FLOAT_STATE_COUNT}, 
-		{PPC_EXCEPTION_STATE, PPC_EXCEPTION_STATE_COUNT}
+		{PPC_EXCEPTION_STATE, PPC_EXCEPTION_STATE_COUNT},
+		{PPC_VECTOR_STATE, PPC_VECTOR_STATE_COUNT}
 		};
-int mynum_flavors=3;
+int mynum_flavors=4;
 #elif defined (__i386__)
 mythread_state_flavor_t thread_flavor_array [] = { 
 		{i386_THREAD_STATE, i386_THREAD_STATE_COUNT},
@@ -168,7 +172,7 @@ coredump(p)
 	int vbrcount=0;
 	tir_t tir1;
 	struct vnode * vp;
-
+	extern boolean_t coredumpok(vm_map_t map, vm_offset_t va);  /* temp fix */
 
 	if (pcred->p_svuid != pcred->p_ruid || pcred->p_svgid != pcred->p_rgid)
 		return (EFAULT);
@@ -181,14 +185,6 @@ coredump(p)
 		return (EFAULT);
 	(void) task_suspend(task);
 
-	/*
-	 *	Make sure all registers, etc. are in pcb so they get
-	 *	into core file.
-	 */
-#if defined (__ppc__)
-	fpu_save(current_act());
-	vec_save(current_act());
-#endif
 	sprintf(core_name, "/cores/core.%d", p->p_pid);
 	NDINIT(&nd, LOOKUP, FOLLOW, UIO_SYSSPACE, core_name, p);
 	if(error = vn_open(&nd, O_CREAT | FWRITE, S_IRUSR ))
@@ -260,7 +256,7 @@ coredump(p)
 	mh->sizeofcmds = command_size;
 
 	hoffset = sizeof(struct mach_header);	/* offset into header */
-	foffset = round_page(header_size);	/* offset into file */
+	foffset = round_page_32(header_size);	/* offset into file */
 	vmoffset = VM_MIN_ADDRESS;		/* offset into VM */
 	/* We use to check for an error, here, now we try and get 
 	 * as much as we can
@@ -297,6 +293,7 @@ coredump(p)
 		sc->cmd = LC_SEGMENT;
 		sc->cmdsize = sizeof(struct segment_command);
 		/* segment name is zerod by kmem_alloc */
+		sc->segname[0] = 0;
 		sc->vmaddr = vmoffset;
 		sc->vmsize = size;
 		sc->fileoff = foffset;
@@ -318,7 +315,7 @@ coredump(p)
 		 *	Note: if we can't read, then we end up with
 		 *	a hole in the file.
 		 */
-		if ((maxprot & VM_PROT_READ) == VM_PROT_READ && vbr.user_tag != VM_MEMORY_IOKIT) {
+		if ((maxprot & VM_PROT_READ) == VM_PROT_READ && vbr.user_tag != VM_MEMORY_IOKIT && coredumpok(map,vmoffset)) {
 			error = vn_rdwr(UIO_WRITE, vp, (caddr_t)vmoffset, size, foffset,
 				UIO_USERSPACE, IO_NODELOCKED|IO_UNIT, cred, (int *) 0, p);
 		}
