@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -45,7 +48,7 @@
 #include <sys/signal.h>
 #include <sys/tty.h>
 #include <kern/task.h>
-#include <ufs/ufs/quota.h>
+#include <sys/quota.h>
 #include <ufs/ufs/inode.h>
 #if	NCPUS > 1
 #include <kern/processor.h>
@@ -171,13 +174,6 @@ proc_shutdown()
 	if (p && p != self)
 		task_suspend(p->task);		/* stop init */
 
-	/*
-	 * Suspend mach_init
-	 */
-	p = pfind(2);
-	if (p && p != self)
-		task_suspend(p->task);		/* stop mach_init */
-
 	printf("Killing all processes ");
 
 	/*
@@ -251,7 +247,8 @@ proc_shutdown()
 			 * XXX
 			 */
 		        if (p->exit_thread) {	/* someone already doing it */
-			        thread_block(0);/* give him a chance */
+						/* give him a chance */
+			        thread_block(THREAD_CONTINUE_NULL);
 			}
 			else {
 			        p->exit_thread = current_thread();
