@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -69,10 +72,6 @@
 #include <sys/errno.h>
 #include <sys/select.h>
 #include <kern/thread.h>
-#if 0 /* [ */
-#include <kern/sched_prim.h>
-#include <kern/parallel.h>
-#endif  /* 0 ] */
 
 #define LOG_RDPRI	(PZERO + 1)
 
@@ -107,11 +106,9 @@ logopen(dev, flags, mode, p)
 	int flags, mode;
 	struct proc *p;
 {
-	unix_master();			/* for pg_id, sigh */
 	LOG_LOCK();
 	if (log_open) {
 		LOG_UNLOCK();
-		unix_release();
 		return (EBUSY);
 	}
 	log_open = 1;
@@ -130,7 +127,7 @@ logopen(dev, flags, mode, p)
 			msgbufp->msg_bufc[i] = 0;
 	}
 	LOG_UNLOCK();
-	unix_release();
+
 	return (0);
 }
 
@@ -235,7 +232,6 @@ logwakeup()
 	funnel_state = thread_funnel_set(kernel_flock, TRUE);
 	selwakeup(&logsoftc.sc_selp);
 	if (logsoftc.sc_state & LOG_ASYNC) {
-		unix_master();
 		LOG_LOCK();
 		pgid = logsoftc.sc_pgid;
 		LOG_UNLOCK();
@@ -243,7 +239,6 @@ logwakeup()
 			gsignal(-pgid, SIGIO); 
 		else if (p = pfind(pgid))
 			psignal(p, SIGIO);
-		unix_release();
 	}
 	if (logsoftc.sc_state & LOG_RDWAIT) {
 		wakeup((caddr_t)msgbufp);
