@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -71,6 +74,7 @@
 #include <sys/uio.h>
 #include <sys/kernel.h>
 #include <sys/vnode.h>
+#include <sys/user.h>
 #include <sys/signalvar.h>
 
 #ifndef NeXT
@@ -316,12 +320,14 @@ ptsread(dev, uio, flag)
 	register struct tty *tp = pt_tty[minor(dev)];
 	register struct pt_ioctl *pti = &pt_ioctl[minor(dev)];
 	int error = 0;
+	struct uthread *ut;
 
+	ut = (struct uthread *)get_bsdthread_info(current_act());
 again:
 	if (pti->pt_flags & PF_REMOTE) {
 		while (isbackground(p, tp)) {
 			if ((p->p_sigignore & sigmask(SIGTTIN)) ||
-			    (p->p_sigmask & sigmask(SIGTTIN)) ||
+			    (ut->uu_sigmask & sigmask(SIGTTIN)) ||
 			    p->p_pgrp->pg_jobc == 0 ||
 			    p->p_flag & P_PPWAIT)
 				return (EIO);
@@ -558,9 +564,10 @@ ptsstop(tp, flush)
 }
 
 FREE_BSDSTATIC int
-ptcselect(dev, rw, p)
+ptcselect(dev, rw, wql, p)
 	dev_t dev;
 	int rw;
+	void * wql;
 	struct proc *p;
 {
 	register struct tty *tp = pt_tty[minor(dev)];
@@ -589,7 +596,7 @@ ptcselect(dev, rw, p)
 		    ((pti->pt_flags&PF_PKT && pti->pt_send) ||
 		     (pti->pt_flags&PF_UCNTL && pti->pt_ucntl)))
 			return (1);
-		selrecord(p, &pti->pt_selr);
+		selrecord(p, &pti->pt_selr, wql);
 		break;
 
 
@@ -605,7 +612,7 @@ ptcselect(dev, rw, p)
 				    return (1);
 			}
 		}
-		selrecord(p, &pti->pt_selw);
+		selrecord(p, &pti->pt_selw, wql);
 		break;
 
 	}
