@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -54,15 +57,8 @@
  *	@(#)random.c	8.1 (Berkeley) 6/10/93
  */
 
-/*
- * Modification History
- *
- * Feb 22, 1999 	Dieter Siegmund (dieter@apple.com)
- * - the first time, set the random seed to the microsecond time value
- *   to make the random numbers less predictable
- */
-#include <libkern/libkern.h>
-#include <sys/time.h>
+#include <sys/types.h>
+#include <sys/random.h>
 
 /*
  * Pseudo-random number generator for randomizing the profiling clock,
@@ -72,32 +68,6 @@
 u_long
 random()
 {
-    	static int first = 1;
-    	static u_long randseed = 1;
-
-	register long x, hi, lo, t;
-
-	if (first) {
-	    struct timeval tv;
-	    microtime(&tv);
-	    randseed = tv.tv_usec;
-	    if(!randseed)
-		randseed=1;
-	    first = 0;
-	}
-
-	/*
-	 * Compute x[n + 1] = (7^5 * x[n]) mod (2^31 - 1).
-	 * From "Random number generators: good ones are hard to find",
-	 * Park and Miller, Communications of the ACM, vol. 31, no. 10,
-	 * October 1988, p. 1195.
-	 */
-	x = randseed;
-	hi = x / 127773;
-	lo = x % 127773;
-	t = 16807 * lo - 2836 * hi;
-	if (t <= 0)
-		t += 0x7fffffff;
-	randseed = t;
-	return (t);
+	return RandomULong();
 }
+
