@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -83,19 +86,16 @@
 #include <miscfs/fdesc/fdesc.h>
 #include <vfs/vfs_support.h>
 
-#define cttyvp(p) ((p)->p_flag & P_CONTROLT ? (p)->p_session->s_ttyvp : NULL)
-
 #define FDL_WANT	0x01
 #define FDL_LOCKED	0x02
 static int fdcache_lock;
 
-dev_t devctty;
 
 #if (FD_STDIN != FD_STDOUT-1) || (FD_STDOUT != FD_STDERR-1)
 FD_STDIN, FD_STDOUT, FD_STDERR must be a sequence n, n+1, n+2
 #endif
 
-#define	NFDCACHE 4
+#define	NFDCACHE 3
 
 #define FD_NHASH(ix) \
 	(&fdhashtbl[(ix) & fdhash])
@@ -109,7 +109,6 @@ fdesc_init(vfsp)
 	struct vfsconf *vfsp;
 {
 
-	devctty = makedev(nchrdev, 0);
 	fdhashtbl = hashinit(NFDCACHE, M_CACHE, &fdhash);
 }
 
@@ -207,7 +206,6 @@ fdesc_lookup(ap)
 	default:
 	case Flink:
 	case Fdesc:
-	case Fctty:
 		error = ENOTDIR;
 		goto bad;
 
@@ -222,21 +220,6 @@ fdesc_lookup(ap)
 			return (0);
 		}
 
-		if (cnp->cn_namelen == 3 && bcmp(pname, "tty", 3) == 0) {
-			struct vnode *ttyvp = cttyvp(p);
-			if (ttyvp == NULL) {
-				error = ENXIO;
-				goto bad;
-			}
-			error = fdesc_allocvp(Fctty, FD_CTTY, dvp->v_mount, &fvp);
-			if (error)
-				goto bad;
-			*vpp = fvp;
-			fvp->v_type = VCHR;
-			vn_lock(fvp, LK_SHARED | LK_RETRY, p);
-			return (0);
-		}
-
 		ln = 0;
 		switch (cnp->cn_namelen) {
 		case 5:
@@ -340,9 +323,6 @@ fdesc_open(ap)
 		error = ENODEV;
 		break;
 
-	case Fctty:
-		error = cttyopen(devctty, ap->a_mode, 0, ap->a_p);
-		break;
 	}
 
 	return (error);
@@ -397,7 +377,7 @@ fdesc_attr(fd, vap, cred, p)
 		break;
 
 	default:
-		panic("fdesc attr");
+		return (EBADF);
 		break;
 	}
 
@@ -422,7 +402,6 @@ fdesc_getattr(ap)
 	case Froot:
 	case Fdevfd:
 	case Flink:
-	case Fctty:
 		bzero((caddr_t) vap, sizeof(*vap));
 		vattr_null(vap);
 		vap->va_fileid = VTOFDESC(vp)->fd_ix;
@@ -448,14 +427,6 @@ fdesc_getattr(ap)
 			vap->va_size = strlen(VTOFDESC(vp)->fd_link);
 			break;
 
-		case Fctty:
-			vap->va_mode = S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH;
-			vap->va_type = VCHR;
-			vap->va_nlink = 1;
-			vap->va_size = 0;
-			vap->va_rdev = devctty;
-			break;
-
 		default:
 			vap->va_mode = S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH;
 			vap->va_type = VDIR;
@@ -471,7 +442,7 @@ fdesc_getattr(ap)
 		break;
 
 	default:
-		panic("fdesc_getattr");
+		return (EBADF);
 		break;	
 	}
 
@@ -502,9 +473,6 @@ fdesc_setattr(ap)
 	case Fdesc:
 		break;
 
-	case Fctty:
-		return (0);
-
 	default:
 		return (EACCES);
 	}
@@ -546,7 +514,6 @@ static struct dirtmp {
 	{ FD_STDIN, UIO_MX, 5, "stdin" },
 	{ FD_STDOUT, UIO_MX, 6, "stdout" },
 	{ FD_STDERR, UIO_MX, 6, "stderr" },
-	{ FD_CTTY, UIO_MX, 3, "tty" },
 	{ 0 }
 };
 
@@ -570,12 +537,9 @@ fdesc_readdir(ap)
 	 * requests do not need cookies.
 	 */
 	if (ap->a_ncookies)
-		panic("fdesc_readdir: not hungry");
+		return (EINVAL);
 
 	switch (VTOFDESC(ap->a_vp)->fd_type) {
-	case Fctty:
-		return (0);
-
 	case Fdesc:
 		return (ENOTDIR);
 
@@ -601,11 +565,6 @@ fdesc_readdir(ap)
 			i++;
 			
 			switch (dt->d_fileno) {
-			case FD_CTTY:
-				if (cttyvp(uio->uio_procp) == NULL)
-					continue;
-				break;
-
 			case FD_STDIN:
 			case FD_STDOUT:
 			case FD_STDERR:
@@ -695,19 +654,8 @@ fdesc_read(ap)
 		struct ucred *a_cred;
 	} */ *ap;
 {
-	int error = EOPNOTSUPP;
 
-	switch (VTOFDESC(ap->a_vp)->fd_type) {
-	case Fctty:
-		error = cttyread(devctty, ap->a_uio, ap->a_ioflag);
-		break;
-
-	default:
-		error = EOPNOTSUPP;
-		break;
-	}
-	
-	return (error);
+	return (EOPNOTSUPP);
 }
 
 int
@@ -719,19 +667,7 @@ fdesc_write(ap)
 		struct ucred *a_cred;
 	} */ *ap;
 {
-	int error = EOPNOTSUPP;
-
-	switch (VTOFDESC(ap->a_vp)->fd_type) {
-	case Fctty:
-		error = cttywrite(devctty, ap->a_uio, ap->a_ioflag);
-		break;
-
-	default:
-		error = EOPNOTSUPP;
-		break;
-	}
-	
-	return (error);
+	return (EOPNOTSUPP);
 }
 
 int
@@ -745,20 +681,7 @@ fdesc_ioctl(ap)
 		struct proc *a_p;
 	} */ *ap;
 {
-	int error = EOPNOTSUPP;
-
-	switch (VTOFDESC(ap->a_vp)->fd_type) {
-	case Fctty:
-		error = cttyioctl(devctty, ap->a_command, ap->a_data,
-					ap->a_fflag, ap->a_p);
-		break;
-
-	default:
-		error = EOPNOTSUPP;
-		break;
-	}
-	
-	return (error);
+	return (EOPNOTSUPP);
 }
 
 int
@@ -772,19 +695,7 @@ fdesc_select(ap)
 		struct proc *a_p;
 	} */ *ap;
 {
-	int error = EOPNOTSUPP;
-
-	switch (VTOFDESC(ap->a_vp)->fd_type) {
-	case Fctty:
-		error = cttyselect(devctty, ap->a_fflags, ap->a_wql, ap->a_p);
-		break;
-
-	default:
-		error = EOPNOTSUPP;
-		break;
-	}
-	
-	return (error);
+	return (EOPNOTSUPP);
 }
 
 int
@@ -892,7 +803,7 @@ int
 fdesc_badop()
 {
 
-	panic("fdesc: bad op");
+	return (ENOTSUP);
 	/* NOTREACHED */
 }
 
