@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -71,6 +74,7 @@
 #include <net/if_dl.h>
 #include <net/if_types.h>
 #include <netinet/if_ether.h>
+#include <netinet/in.h>	/* For M_LOOP */
 
 /*
 #if INET
@@ -109,7 +113,6 @@ extern struct ifqueue atalkintrq;
 #endif /* NVLAN > 0 */
 
 static u_long lo_dlt = 0;
-static ivedonethis = 0;
 
 #define IFP2AC(IFP) ((struct arpcom *)IFP)
 
@@ -249,7 +252,7 @@ ether_add_proto(struct ddesc_head_str *desc_head, struct if_proto *proto, u_long
         }
         
         ed[i].proto	= proto;
-        ed[i].data[0]	= 0;
+        ed[i].data[0] = 0;
         ed[i].data[1] = 0;
         
         switch (desc->type) {
@@ -291,7 +294,7 @@ ether_add_proto(struct ddesc_head_str *desc_head, struct if_proto *proto, u_long
             case DLIL_DESC_SNAP: {
                 u_int8_t*	pDest = ((u_int8_t*)&ed[i].data[0]) + 3;
                 ed[i].type = DLIL_DESC_SNAP;
-                bcopy(&desc->native_type, pDest, 5);
+                bcopy(desc->native_type, pDest, 5);
             }
             break;
         }
@@ -332,6 +335,24 @@ int ether_demux(ifp, m, frame_header, proto)
             m->m_flags |= M_BCAST;
         else
             m->m_flags |= M_MCAST;
+    } else {
+        /*
+         * When the driver is put into promiscuous mode we may receive unicast
+         * frames that are not intended for our interfaces.  They are filtered
+         * here to keep them from traveling further up the stack to code that
+         * is not expecting them or prepared to deal with them.  In the near
+         * future, the filtering done here will be moved even further down the
+         * stack into the IONetworkingFamily, preventing even interface
+         * filter NKE's from receiving promiscuous packets.  Please use BPF.
+         */
+        #define ETHER_CMP(x, y) ( ((u_int16_t *) x)[0] != ((u_int16_t *) y)[0] || \
+                                  ((u_int16_t *) x)[1] != ((u_int16_t *) y)[1] || \
+                                  ((u_int16_t *) x)[2] != ((u_int16_t *) y)[2] )
+    
+        if (ETHER_CMP(eh->ether_dhost, ((struct arpcom *) ifp)->ac_enaddr)) {
+            m_freem(m);
+            return EJUSTRETURN;
+        }
     }
     
     data = mtod(m, u_int8_t*);
@@ -341,7 +362,7 @@ int ether_demux(ifp, m, frame_header, proto)
      * longs for quick compares.
      */
     
-    if (ntohs(ether_type) < 1500) {
+    if (ntohs(ether_type) <= 1500) {
         extProto1 = *(u_int32_t*)data;
         
         // SAP or SNAP
@@ -516,6 +537,24 @@ int  ether_del_if(struct ifnet *ifp)
         return ENOENT;
 }
 
+static
+int  ether_init_if(struct ifnet *ifp)
+{
+    register struct ifaddr *ifa;
+    register struct sockaddr_dl *sdl;
+
+    ifa = ifnet_addrs[ifp->if_index - 1];
+    if (ifa == 0) {
+            printf("ether_ifattach: no lladdr!\n");
+            return;
+    }
+    sdl = (struct sockaddr_dl *)ifa->ifa_addr;
+    sdl->sdl_type = IFT_ETHER;
+    sdl->sdl_alen = ifp->if_addrlen;
+    bcopy((IFP2AC(ifp))->ac_enaddr, LLADDR(sdl), ifp->if_addrlen);
+
+    return 0;
+}
 
 
 int
@@ -571,13 +610,13 @@ int ether_family_init()
     int  i;
     struct dlil_ifmod_reg_str  ifmod_reg;
 
-    if (ivedonethis)
-        return 0;
-
-    ivedonethis = 1;
+    /* ethernet family is built-in, called from bsd_init */
+    thread_funnel_switch(KERNEL_FUNNEL, NETWORK_FUNNEL);
 
+    bzero(&ifmod_reg, sizeof(ifmod_reg));
     ifmod_reg.add_if = ether_add_if;
     ifmod_reg.del_if = ether_del_if;
+    ifmod_reg.init_if = ether_init_if;
     ifmod_reg.add_proto = ether_add_proto;
     ifmod_reg.del_proto = ether_del_proto;
     ifmod_reg.ifmod_ioctl = ether_ifmod_ioctl;
@@ -591,5 +630,7 @@ int ether_family_init()
     for (i=0; i < MAX_INTERFACES; i++)
         ether_desc_blk[i].n_count = 0;
 
+    thread_funnel_switch(NETWORK_FUNNEL, KERNEL_FUNNEL);
+
     return 0;
 }
