@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -237,7 +240,7 @@ inet_ether_pre_output(ifp, m0, dst_netaddr, route, type, edst, dl_tag )
 	if ((rt->rt_flags & RTF_UP) == 0) {
 	    rt0 = rt = rtalloc1(dst_netaddr, 1, 0UL);
 	    if (rt0)
-		rt->rt_refcnt--;
+		rtunref(rt);
 	    else
 		return EHOSTUNREACH;
 	}
@@ -377,6 +380,13 @@ ether_inet_prmod_ioctl(dl_tag, ifp, command, data)
 
 	    arp_ifinit(IFP2AC(ifp), ifa);
 
+	    /*
+	     * Register new IP and MAC addresses with the kernel debugger
+	     * for the en0 interface.
+	     */
+	    if (ifp->if_unit == 0)
+		kdp_set_ip_and_mac_addresses(&(IA_SIN(ifa)->sin_addr), &(IFP2AC(ifp)->ac_enaddr));
+
 	    break;
 
 	default:
@@ -397,14 +407,10 @@ ether_inet_prmod_ioctl(dl_tag, ifp, command, data)
 
     case SIOCSIFMTU:
 	/*
-	 * Set the interface MTU.
+	 * IOKit IONetworkFamily will set the right MTU according to the driver
 	 */
-	if (ifr->ifr_mtu > ETHERMTU) {
-	    error = EINVAL;
-	} else {
-	    ifp->if_mtu = ifr->ifr_mtu;
-	}
-	break;
+
+	 return (0);
 
     default:
 	 return EOPNOTSUPP;
@@ -462,8 +468,6 @@ ether_attach_inet(struct ifnet *ifp)
 	printf("WARNING: ether_attach_inet can't attach ip to interface\n");
 	return stat;
     }
-    /* XXX avoid free'ing the interface */
-    ifp->if_eflags |= IFEF_DETACH_DISABLED;
     return ip_dl_tag;
 }
 
