@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -23,6 +26,8 @@
  *	Copyright (c) 1998 Apple Computer, Inc. 
  */
 
+#include <sys/appleapiopts.h>
+#ifdef __APPLE_API_PRIVATE
 #include <sys/queue.h>
 
 /* at_var.h */
@@ -277,3 +282,27 @@ int ddp_ctloutput __P((struct socket *, struct sockopt *));
 void ddp_init __P((void));;
 void ddp_slowtimo __P((void));
 #endif
+
+/*
+ * Define AppleTalk event subclass and specific AppleTalk events.
+ */
+
+#define KEV_ATALK_SUBCLASS 5
+
+#define KEV_ATALK_ENABLED				1	/* AppleTalk enabled from user space - node/net set and valid */
+#define KEV_ATALK_DISABLED				2	/* AppleTalk disabled from user space */
+#define KEV_ATALK_ZONEUPDATED			3	/* Zone for this node set/changed */
+#define KEV_ATALK_ROUTERUP				4	/* Seed router found with valid cable range */
+#define KEV_ATALK_ROUTERUP_INVALID		5	/* Seed router found with invalid cable range */
+#define KEV_ATALK_ROUTERDOWN			6	/* Seed router down */
+#define KEV_ATALK_ZONELISTCHANGED		7	/* Zone list changed by router */
+
+struct kev_atalk_data {
+	struct	net_event_data	link_data;
+	union {
+		struct	at_addr		address;
+		at_nvestr_t			zone;
+	} node_data;
+};
+
+#endif /* __APPLE_API_PRIVATE */
