@@ -1,21 +1,24 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2002 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -393,10 +396,12 @@ int _ATrw(fp, rw, uio, ext)
   return 0;
 } /* _ATrw */
 
-int _ATread(fp, uio, cred)
+int _ATread(fp, uio, cred, flags, p)
 	void *fp;
 	struct uio *uio;
 	void *cred;
+	int flags;
+	struct proc *p;
 {
      int stat;
 
@@ -406,10 +411,12 @@ int _ATread(fp, uio, cred)
 	return stat;
 }
 
-int _ATwrite(fp, uio, cred)
+int _ATwrite(fp, uio, cred, flags, p)
 	void *fp;
 	struct uio *uio;
 	void *cred;
+	int flags;
+	struct proc *p;
 {
      int stat;
 
@@ -650,7 +657,7 @@ void atalk_putnext(gref, m)
 #ifdef APPLETALK_DEBUG
 				kprintf("wakeup gref = 0x%x\n", (unsigned)gref);
 #endif
-				thread_wakeup(&gref->iocevent);
+				wakeup(&gref->iocevent);
 			}
 		}
 		break;
@@ -669,7 +676,7 @@ void atalk_putnext(gref, m)
 			gref->rdhead = m;
 			if (gref->sevents & POLLMSG) {
 				gref->sevents &= ~POLLMSG;
-				thread_wakeup(&gref->event);
+				wakeup(&gref->event);
 			}
 			if (gref->sevents & POLLIN) {
 				gref->sevents &= ~POLLIN;
@@ -686,7 +693,7 @@ void atalk_enablew(gref)
 	gref_t *gref;
 {
 	if (gref->sevents & POLLSYNC)
-		thread_wakeup(&gref->event);
+		wakeup(&gref->event);
 }
 
 void atalk_flush(gref)
@@ -737,7 +744,7 @@ void atalk_notify(gref, errno)
 		/* blocked read */
 		if (gref->sevents & POLLMSG) {
 			gref->sevents &= ~POLLMSG;
-			thread_wakeup(&gref->event);
+			wakeup(&gref->event);
 		}
 		/* select */
 		if (gref->sevents & POLLIN) {
