@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -60,6 +63,9 @@
 
 #ifndef _NETINET_IGMP_VAR_H_
 #define _NETINET_IGMP_VAR_H_
+#include <sys/appleapiopts.h>
+
+#ifdef __APPLE_API_UNSTABLE
 
 /*
  * Internet Group Management Protocol (IGMP),
@@ -83,6 +89,7 @@ struct igmpstat {
 };
 
 #ifdef KERNEL
+#ifdef __APPLE_API_PRIVATE
 #define IGMP_RANDOM_DELAY(X) (random() % (X) + 1)
 
 /*
@@ -114,6 +121,7 @@ void	igmp_slowtimo __P((void));
 
 SYSCTL_DECL(_net_inet_igmp);
 
+#endif /* __APPLE_API_PRIVATE */
 #endif
 
 /*
@@ -127,6 +135,6 @@ SYSCTL_DECL(_net_inet_igmp);
 	{ "stats", CTLTYPE_STRUCT }, \
 }
 
+#endif /* __APPLE_API_UNSTABLE */
 #endif
 
-
