@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -81,6 +84,7 @@
 
 #include <sys/time.h>
 #include <kern/clock.h>
+#include <sys/user.h>
 
 #include <netinet/in.h>
 #include <netinet/tcp.h>
@@ -984,7 +988,7 @@ nfs_reply(myrep)
 		/*
 		 * Bailout asap if nfsmount struct gone (unmounted). 
 		 */
-		if (!myrep->r_nmp) {
+		if (!myrep->r_nmp || !nmp->nm_so) {
 			FSDBG(530, myrep->r_xid, myrep, nmp, -2);
 			return (ECONNABORTED);
 		}
@@ -992,6 +996,10 @@ nfs_reply(myrep)
 			FSDBG(530, myrep->r_xid, myrep, nmp, error);
 			nfs_rcvunlock(&nmp->nm_flag);
 
+			/* Bailout asap if nfsmount struct gone (unmounted). */
+			if (!myrep->r_nmp || !nmp->nm_so)
+				return (ECONNABORTED);
+
 			/*
 			 * Ignore routing errors on connectionless protocols??
 			 */
@@ -1256,7 +1264,7 @@ kerbauth:
 	m = nfsm_rpchead(cred, nmp->nm_flag, procnum, auth_type, auth_len,
 	     auth_str, verf_len, verf_str, mrest, mrest_len, &mheadend, &xid);
 	if (xidp)
-		*xidp = xid + ((u_int64_t)nfs_xidwrap << 32);
+		*xidp = ntohl(xid) + ((u_int64_t)nfs_xidwrap << 32);
 	if (auth_str)
 		_FREE(auth_str, M_TEMP);
 
@@ -1918,12 +1926,16 @@ nfs_sigintr(nmp, rep, p)
 	register struct proc *p;
 {
 
+	struct uthread *ut;
+
+	ut = (struct uthread *)get_bsdthread_info(current_act());
+
 	if (rep && (rep->r_flags & R_SOFTTERM))
 		return (EINTR);
 	if (!(nmp->nm_flag & NFSMNT_INT))
 		return (0);
-	if (p && p->p_siglist &&
-	    (((p->p_siglist & ~p->p_sigmask) & ~p->p_sigignore) &
+	if (p && ut &&  ut->uu_siglist &&
+	    (((ut->uu_siglist & ~ut->uu_sigmask) & ~p->p_sigignore) &
 	    NFSINT_SIGMASK))
 		return (EINTR);
 	return (0);
@@ -1991,9 +2003,17 @@ static int
 nfs_rcvlock(rep)
 	register struct nfsreq *rep;
 {
-	register int *flagp = &rep->r_nmp->nm_flag;
+	register int *flagp;
 	int slpflag, slptimeo = 0;
 
+	/* make sure we still have our mountpoint */
+	if (!rep->r_nmp) {
+		if (rep->r_mrep != NULL)
+			return (EALREADY);
+		return (ECONNABORTED);
+	}
+
+	flagp = &rep->r_nmp->nm_flag;
 	FSDBG_TOP(534, rep->r_xid, rep, rep->r_nmp, *flagp);
 	if (*flagp & NFSMNT_INT)
 		slpflag = PCATCH;
@@ -2199,7 +2219,7 @@ nfsrv_getstream(slp, waitflag)
 	register struct mbuf *m, **mpp;
 	register char *cp1, *cp2;
 	register int len;
-	struct mbuf *om, *m2, *recm = 0;
+	struct mbuf *om, *m2, *recm;
 	u_long recmark;
 
 	if (slp->ns_flag & SLP_GETSTREAM)
@@ -2244,7 +2264,11 @@ nfsrv_getstream(slp, waitflag)
 
 	    /*
 	     * Now get the record part.
+	     *
+	     * Note that slp->ns_reclen may be 0.  Linux sometimes
+	     * generates 0-length RPCs
 	     */
+	    recm = NULL;
 	    if (slp->ns_cc == slp->ns_reclen) {
 		recm = slp->ns_raw;
 		slp->ns_raw = slp->ns_rawend = (struct mbuf *)0;
