@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -850,7 +853,7 @@ nfssvc_iod(p)
 	/* stuff myiod into uthread to get off local stack for
        continuation */
 
-	ut = get_bsdthread_info(current_act());
+	ut = (struct uthread *)get_bsdthread_info(current_act());
 	ut->uu_state.uu_nfs_myiod = myiod;  /* squirrel away for continuation */
 
 	nfssvc_iod_continue(0);
@@ -873,7 +876,7 @@ nfssvc_iod_continue(error)
 	/*
 	 * real myiod is stored in uthread, recover it
 	 */
-	ut = get_bsdthread_info(current_act());
+	ut = (struct uthread *)get_bsdthread_info(current_act());
 	myiod = ut->uu_state.uu_nfs_myiod;
 	p = current_proc();
 
@@ -901,11 +904,7 @@ nfssvc_iod_continue(error)
 		nfs_numasync--;
 		if (error == EINTR || error == ERESTART)
 		  error = 0;
-#if defined (__i386__)
-		return(error);
-#else
 		unix_syscall_return(error);
-#endif
 	    }
 	    while ((bp = nmp->nm_bufq.tqh_first) != NULL) {
 		/* Take one off the front of the list */
