@@ -1,21 +1,24 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2002 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -62,15 +65,19 @@
 
 #ifndef _SYS_BUF_H_
 #define	_SYS_BUF_H_
+
+#include <sys/appleapiopts.h>
+
+#ifdef KERNEL
 #include <sys/queue.h>
 #include <sys/errno.h>
-
 #include <sys/vm.h>
+#include <sys/cdefs.h>
+
+#ifdef __APPLE_API_PRIVATE
 
 #define NOLIST ((struct buf *)0x87654321)
 
-#include <sys/cdefs.h>
-
 /*
  * The buffer header describes an I/O operation in the kernel.
  */
@@ -128,6 +135,15 @@ struct buf {
 #define b_trans_head b_freelist.tqe_prev
 #define b_trans_next b_freelist.tqe_next
 #define b_real_bp    b_saveaddr
+#define b_iostate    b_rcred
+
+/* journaling uses this cluster i/o field for its own
+ * purposes because meta data buf's should never go
+ * through the clustering code.
+ */
+#define b_transaction b_vectorlist
+
+   
 
 /*
  * These flags are kept in b_flags.
@@ -159,7 +175,7 @@ struct buf {
 #define	B_WRITE		0x00000000	/* Write buffer (pseudo flag). */
 #define	B_WRITEINPROG	0x01000000	/* Write in progress. */
 #define	B_HDRALLOC	0x02000000	/* zone allocated buffer header */
-#define	B_UNUSED1	0x04000000	/* Unused bit */
+#define	B_NORELSE	0x04000000	/* don't brelse() in bwrite() */
 #define B_NEED_IODONE   0x08000000
 								/* need to do a biodone on the */
 								/* real_bp associated with a cluster_io */
@@ -193,15 +209,20 @@ struct buf {
 #define	BLK_CLREAD	0x20	/* buffer for cluster read */
 #define	BLK_CLWRITE	0x40	/* buffer for cluster write */
 
-#ifdef KERNEL
 extern int nbuf;			/* The number of buffer headers */
 extern struct buf *buf;		/* The buffer headers. */
 
+#endif /* __APPLE_API_PRIVATE */
+
+
+#ifdef __APPLE_API_UNSTABLE
 /* Macros to clear/set/test flags. */
 #define	SET(t, f)	(t) |= (f)
 #define	CLR(t, f)	(t) &= ~(f)
 #define	ISSET(t, f)	((t) & (f))
+#endif /* __APPLE_API_UNSTABLE */
 
+#ifdef __APPLE_API_PRIVATE
 /*
  * Definitions for the buffer free lists.
  */
@@ -213,8 +234,10 @@ extern struct buf *buf;		/* The buffer headers. */
 #define	BQ_EMPTY	3		/* buffer headers with no memory */
 #define BQ_META		4		/* buffer containing metadata */
 #define BQ_LAUNDRY	5		/* buffers that need cleaning */
+#endif /* __APPLE_API_PRIVATE */
 
 __BEGIN_DECLS
+#ifdef __APPLE_API_UNSTABLE
 int	allocbuf __P((struct buf *, int));
 void	bawrite __P((struct buf *));
 void	bdwrite __P((struct buf *));
@@ -231,6 +254,7 @@ int	breadn __P((struct vnode *, daddr_t, int, daddr_t *, int *, int,
 void	brelse __P((struct buf *));
 void	bremfree __P((struct buf *));
 void	bufinit __P((void));
+void	bwillwrite __P((void));
 int	bwrite __P((struct buf *));
 struct buf *getblk __P((struct vnode *, daddr_t, int, int, int, int));
 struct buf *geteblk __P((int));
@@ -240,8 +264,11 @@ int physio __P((void (*)(struct buf *), struct buf *, dev_t, int ,  u_int (*)(st
 int count_busy_buffers __P((void));
 struct buf *alloc_io_buf __P((struct vnode *, int));
 void free_io_buf __P((struct buf *));
+void reassignbuf __P((struct buf *, struct vnode *));
+#endif /* __APPLE_API_UNSTABLE */
 __END_DECLS
 
+#ifdef __APPLE_API_PRIVATE
 /*
  *	Stats on usefulness of the buffer cache
  */
@@ -256,6 +283,7 @@ struct bufstats {
 	long	bufs_iobufinuse;	/* number of IO buffers in use */
 	long	bufs_iobufsleeps;	/* IO buffer starvation */
 };
+#endif /* __APPLE_API_PRIVATE */
 
 #endif /* KERNEL */
 #endif /* !_SYS_BUF_H_ */
