@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -63,6 +66,8 @@
 #include <sys/file.h>
 #include <sys/vnode.h>
 #include <sys/ubc.h>
+#include <sys/quota.h>
+
 #if REV_ENDIAN_FS
 #include <sys/mount.h>
 #endif /* REV_ENDIAN_FS */
@@ -421,7 +426,7 @@ fail:
 		/*
 		 * Restore user's disk quota because allocation failed.
 		 */
-		(void) chkdq(ip, (long)-btodb(deallocated, devBlockSize), cred, FORCE);
+		(void) chkdq(ip, (int64_t)-deallocated, cred, FORCE);
 #endif /* QUOTA */
 		ip->i_blocks -= btodb(deallocated, devBlockSize);
 		ip->i_flag |= IN_CHANGE | IN_UPDATE;
@@ -666,7 +671,7 @@ fail:
 		/*
 		 * Restore user's disk quota because allocation failed.
 		 */
-		(void) chkdq(ip, (long)-btodb(deallocated, devBlockSize), cred, FORCE);
+		(void) chkdq(ip, (int64_t)-deallocated, cred, FORCE);
 #endif /* QUOTA */
 		ip->i_blocks -= btodb(deallocated, devBlockSize);
 		ip->i_flag |= IN_CHANGE | IN_UPDATE;
