@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -69,6 +72,7 @@
 #include <sys/vnode.h>
 #include <sys/malloc.h>
 #include <sys/ubc.h>
+#include <sys/quota.h>
 
 #include <sys/vm.h>
 #include <vfs/vfs_support.h>
@@ -295,6 +299,7 @@ ffs_fsync(ap)
 	struct buf *nbp;
 	int s;
 	struct inode *ip = VTOI(vp);
+	int retry = 0;
 
 	/*
 	 * Write out any clusters.
@@ -325,8 +330,6 @@ loop:
 			(void) bwrite(bp);
 		goto loop;
 	}
-	if (vp->v_flag & VHASDIRTY)
-		ubc_pushdirty(vp);
 
 	if (ap->a_waitfor == MNT_WAIT) {
 		while (vp->v_numoutput) {
@@ -334,12 +337,25 @@ loop:
 			tsleep((caddr_t)&vp->v_numoutput, PRIBIO + 1, "ffs_fsync", 0);
 		}
 
-		/* I have seen this happen for swapfile. So it is safer to
-		 * check for dirty buffers again.  --Umesh
-		 */
-		if (vp->v_dirtyblkhd.lh_first || (vp->v_flag & VHASDIRTY)) {
-			vprint("ffs_fsync: dirty", vp);
-			splx(s);
+		if (vp->v_dirtyblkhd.lh_first) {
+			/* still have some dirty buffers */
+			if (retry++ > 10) {
+				vprint("ffs_fsync: dirty", vp);
+				splx(s);
+				/*
+				 * Looks like the requests are not
+				 * getting queued to the driver.
+				 * Retrying here causes a cpu bound loop.
+				 * Yield to the other threads and hope
+				 * for the best.
+				 */
+				(void)tsleep((caddr_t)&vp->v_numoutput,
+						PRIBIO + 1, "ffs_fsync", hz/10);
+				retry = 0;
+			} else {
+				splx(s);
+			}
+			/* try again */
 			goto loop;
 		}
 	}
