@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -62,6 +65,7 @@ private:
 
     enum {			          // when kPMUADBint is set
         kPMUADBint          = 0x10,
+        kPMUenvironmentInt  = 0x40,	  // Environment changed (clamshell)
         kPMUwaitinglsc	    = 0x01,       // waiting to listen to charger
         kPMUautoSRQpolling  = 0x02,	  // auto/SRQ polling is enabled
         kPMUautopoll	    = 0x04	  // input is autopoll data
@@ -81,6 +85,7 @@ private:
     IOService	 	*PMUdriver;
     UInt32		pollList;		// ADB autopoll device bitmap
     bool		autopollOn;		// TRUE: PMU is autopolling
+    bool		clamshellOpen;		// Normally TRUE
 
     UInt32		dataLen;		// data len as result of an interrupt
     UInt8		dataBuffer[256];	// data as result of an interrupt
