@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -101,7 +104,7 @@ void IOConditionLock::unlock()
     want_lock = false;
     if (waiting) {
 	waiting = false;
-	thread_wakeup(this);	// Wakeup everybody
+	IOLockWakeup(sleep_interlock, this, /* one-thread */ false);	// Wakeup everybody
     }
 
     IOUnlock(sleep_interlock);
@@ -145,12 +148,7 @@ int IOConditionLock::lock()
     while (want_lock && thread_res == THREAD_AWAKENED)
     {
 	waiting = true;
-
-        assert_wait((void *) this, interruptible);	/* assert event */
-	IOUnlock(sleep_interlock);			/* release the lock */
-	thread_res = thread_block((void (*)(void)) 0);	/* block ourselves */
-
-        IOTakeLock(sleep_interlock);
+	thread_res = IOLockSleep(sleep_interlock, (void *) this, interruptible);
     }
     if (thread_res == THREAD_AWAKENED)
 	want_lock = true;
@@ -191,8 +189,8 @@ int IOConditionLock::lockWhen(int inCondition)
 	 * of changes in _condition.
 	 */
         assert_wait((void *) &condition, interruptible); /* assert event */
-	IOUnlock(cond_interlock);			/* release the lock */
-	thread_res = thread_block((void (*)(void)) 0);	/* block ourselves */
+	IOUnlock(cond_interlock);			 /* release the lock */
+	thread_res = thread_block(THREAD_CONTINUE_NULL); /* block ourselves */
     } while (thread_res == THREAD_AWAKENED);
     
     return thread_res;
