@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -27,6 +30,10 @@
 #ifndef	_MACH_DEFAULT_PAGER_TYPES_H_
 #define _MACH_DEFAULT_PAGER_TYPES_H_
 
+#include <sys/appleapiopts.h>
+
+#ifdef __APPLE_API_UNSTABLE
+
 #include <mach/mach_types.h>
 #include <mach/machine/vm_types.h>
 #include <mach/memory_object_types.h>
@@ -82,7 +89,6 @@ typedef struct default_pager_object {
 
 typedef default_pager_object_t *default_pager_object_array_t;
 
-
 typedef struct default_pager_page {
 	vm_offset_t dpp_offset;		/* offset of the page in its object */
 } default_pager_page_t;
@@ -96,4 +102,6 @@ typedef default_pager_page_t *default_pager_page_array_t;
 #define HI_WAT_ALERT	1
 #define LO_WAT_ALERT	2
 
+#endif /* __APPLE_API_UNSTABLE */
+
 #endif	/* _MACH_DEFAULT_PAGER_TYPES_H_ */
