@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -113,7 +116,10 @@
 #include <i386/AT386/mp/mp_v1_1.h>
 #endif	/* MP_V1_1 */
 
-vm_size_t	mem_size = 0; 
+#include <IOKit/IOPlatformExpert.h>
+
+vm_size_t	mem_size = 0;
+uint64_t	max_mem;
 vm_offset_t	first_addr = 0;	/* set by start.s - keep out of bss */
 vm_offset_t	first_avail = 0;/* first after page tables */
 vm_offset_t	last_addr;
@@ -138,9 +144,6 @@ extern char	edata, end;
 
 extern char	version[];
 
-int		rebootflag = 0;	/* exported to kdintr */
-
-
 void		parse_arguments(void);
 const char	*getenv(const char *);
 
@@ -208,8 +211,6 @@ machine_startup(void)
 {
 
 #ifdef	__MACHO__
-
-
 	/* Now copy over various bits.. */
 	cnvmem = kernBootStruct->convmem;
 	extmem = kernBootStruct->extmem;
@@ -234,14 +235,6 @@ machine_startup(void)
 	edata = (vm_offset_t) sectDATAB + sectSizeDATA;
 #endif
 
-	/*
-	 * Parse startup arguments
-	 */
-	parse_arguments();
-
-	disableDebugOuput = FALSE;
-	debug_mode = TRUE;
-	
 	printf_init();						/* Init this in case we need debugger */
 	panic_init();						/* Init this in case we need debugger */
 
@@ -249,6 +242,18 @@ machine_startup(void)
 	PE_init_kprintf(FALSE);
 	PE_init_printf(FALSE);
 
+	/*
+	 * Parse startup arguments
+	 */
+	parse_arguments();
+
+	/*
+	 * Set up initial thread so current_thread() works early on
+	 */
+	pageout_thread.top_act = &pageout_act;
+	pageout_act.thread = &pageout_thread;
+	thread_machine_set_current(&pageout_thread);
+
 	/*
 	 * Do basic VM initialization
 	 */
@@ -259,7 +264,6 @@ machine_startup(void)
 	PE_init_printf(TRUE);
 
 #if	MACH_KDB
-
 	/*
 	 * Initialize the kernel debugger.
 	 */
@@ -269,9 +273,6 @@ machine_startup(void)
 	 * Cause a breakpoint trap to the debugger before proceeding
 	 * any further if the proper option bit was specified in
 	 * the boot flags.
-	 *
-	 * XXX use -a switch to invoke kdb, since there's no
-	 *     boot-program switch to turn on RB_HALT!
 	 */
 
 	if (halt_in_debugger) {
@@ -279,6 +280,7 @@ machine_startup(void)
 	        Debugger("inline call");
 	}
 #endif	/* MACH_KDB */
+
 	TR_INIT();
 
 	printf(version);
@@ -314,34 +316,18 @@ vm_size_t	env_size = 0;		/* size of environment */
 void
 parse_arguments(void)
 {
-	char *p = (char *) kern_args_start;
-	char *endp = (char *) kern_args_start + kern_args_size - 1;
-	char ch;
+    unsigned int boot_arg;
 
-	if (kern_args_start == 0)
-	    return;
-
-	/*
-	 * handle switches in exact format of  -h  or -m64
-	 */
-	while ( (p < endp) && (*p != '\0')) {
-	  if (*p++ != '-') 
-	    continue;
-	  switch (*p++) {
-	  case 'h':
-	    halt_in_debugger = 1;
-	    break;
-	  case 'm':
-	    mem_size = atoi_term(p,&p)*1024*1024;
-	    break;
-	  case 'k':
-	    mem_size = atoi_term(p,&p)*1024;
-	    break;
-	  default:
-	    break;
-	  }
-	}
+    if (PE_parse_boot_arg("maxmem", &boot_arg))
+    {
+        mem_size = boot_arg * (1024 * 1024);
+    }
 
+    if (PE_parse_boot_arg("debug", &boot_arg))
+    {
+        if (boot_arg & DB_HALT) halt_in_debugger  = 1;
+        if (boot_arg & DB_PRT)  disableDebugOuput = FALSE;
+    }
 }
 
 const char *
@@ -385,7 +371,7 @@ machine_init(void)
 	 * Display CPU identification
 	 */
 	cpuid_cpu_display("CPU identification", 0);
-	cpuid_cache_display("CPU configuration", 0);
+	cpuid_feature_display("CPU features", 0);
 
 #if	MP_V1_1
 	mp_v1_1_init();
@@ -423,25 +409,25 @@ int reset_mem_on_reboot = 1;
  * Halt the system or reboot.
  */
 void
-halt_all_cpus(
-	boolean_t	reboot)
+halt_all_cpus(boolean_t reboot)
 {
-	if (reboot) {
-	    /*
-	     * Tell the BIOS not to clear and test memory.
-	     */
-	    if (! reset_mem_on_reboot)
-		*(unsigned short *)phystokv(0x472) = 0x1234;
+	if (reboot)
+    {
+        /*
+         * Tell the BIOS not to clear and test memory.
+         */
+        if (!reset_mem_on_reboot)
+            *(unsigned short *)phystokv(0x472) = 0x1234;
 
-	    kdreboot();
-	}
-	else {
-	    rebootflag = 1;
-	    printf("In tight loop: hit ctl-alt-del to reboot\n");
-	    (void) spllo();
-	}
-	for (;;)
-	    continue;
+        printf("MACH Reboot\n");
+        PEHaltRestart( kPERestartCPU );
+    }
+    else
+    {
+        printf("CPU halted\n");
+        PEHaltRestart( kPEHaltCPU );
+    }
+    while(1);
 }
 
 /*
@@ -499,8 +485,9 @@ i386_init(void)
 #endif
 #endif
 
-		/* BIOS leaves data in low memory */
+	/* BIOS leaves data in low memory */
 	last_addr = 1024*1024 + extmem*1024;
+
 	/* extended memory starts at 1MB */
        
 	bios_hole_size = 1024*1024 - trunc_page((vm_offset_t)(1024 * cnvmem));
@@ -525,6 +512,7 @@ i386_init(void)
 	first_addr = round_page(first_addr);
 	last_addr = trunc_page(last_addr);
 	mem_size = last_addr - bios_hole_size;
+	max_mem = mem_size;
 
 	avail_start = first_addr;
 	avail_end = last_addr;
