@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -62,7 +65,6 @@
 #include <kern/task.h>
 #include <kern/lock.h>
 #include <kern/thread_act.h>
-#include <kern/thread_pool.h>
 #include <ipc/ipc_space.h>
 #include <ipc/ipc_port.h>
 #include <ipc/ipc_pset.h>
@@ -74,7 +76,6 @@
 #include <i386/seg.h>
 #include <i386/pmap.h>
 #include <i386/tss.h>
-#include <mach/rpc.h>
 #include <vm/vm_map.h>
 #if	NCPUS > 1
 #include <i386/mp_desc.h>
@@ -151,12 +152,6 @@ main(
 
 	DECLARE("MAP_PMAP",	offsetof(vm_map_t, pmap));
 
-	/* XXX Until rpc buffers move from kernel stack to activation */
-	DECLARE("RPC_CLIENT_BUF_SIZE",	 
-			2 * RPC_KBUF_SIZE * sizeof(int) + 
-			RPC_DESC_COUNT * sizeof(rpc_copy_state_data_t) +
-			40 * sizeof(int));
-
 	DECLARE("HOST_NAME",	offsetof(host_t, host_self));
 
 	DECLARE("DISP_MIN",	offsetof(eml_dispatch_t, disp_min));
