@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -252,6 +255,7 @@ char	*pmap_phys_attributes;
  */
 #define	PHYS_MODIFIED	INTEL_PTE_MOD	/* page modified */
 #define	PHYS_REFERENCED	INTEL_PTE_REF	/* page referenced */
+#define PHYS_NCACHE	INTEL_PTE_NCACHE
 
 /*
  *	Amount of virtual memory mapped by one
@@ -552,7 +556,7 @@ pmap_map(
 
 	ps = PAGE_SIZE;
 	while (start < end) {
-		pmap_enter(kernel_pmap, virt, start, prot, FALSE);
+		pmap_enter(kernel_pmap, virt, start, prot, 0, FALSE);
 		virt += ps;
 		start += ps;
 	}
@@ -791,6 +795,19 @@ pmap_bootstrap(
 
 	kernel_pmap->pdirbase = kvtophys((vm_offset_t)kernel_pmap->dirbase);
 
+	if (cpuid_features() & CPUID_FEATURE_PAT)
+	{
+		uint64_t pat;
+		uint32_t msr;
+	    
+		msr = 0x277;
+		asm volatile("rdmsr" : "=A" (pat) : "c" (msr));
+	    
+		pat &= ~(0xfULL << 48);
+		pat |= 0x01ULL << 48;
+	    
+		asm volatile("wrmsr" :: "A" (pat), "c" (msr));
+	}
 }
 
 void
@@ -1284,11 +1301,13 @@ pmap_remove_some_phys(
  *	rounded to the hardware page size.
  */
 
+
+/* FIXMEx86 */
 void
 pmap_remove(
 	pmap_t		map,
-	vm_offset_t	s,
-	vm_offset_t	e)
+	addr64_t	s,
+	addr64_t	e)
 {
 	spl_t			spl;
 	register pt_entry_t	*pde;
@@ -1415,9 +1434,6 @@ pmap_page_protect(
 		    /*
 		     * Remove the mapping, collecting any modify bits.
 		     */
-		    if (iswired(*pte))
-			panic("pmap_remove_all removing a wired page");
-
 		    {
 			register int	i = ptes_per_vm_page;
 
@@ -1584,6 +1600,7 @@ pmap_enter(
 	vm_offset_t		v,
 	register vm_offset_t	pa,
 	vm_prot_t		prot,
+	unsigned int 		flags,
 	boolean_t		wired)
 {
 	register pt_entry_t	*pte;
@@ -1667,8 +1684,15 @@ Retry:
 	    /*
 	     *	May be changing its wired attribute or protection
 	     */
-		
+	
 	    template = pa_to_pte(pa) | INTEL_PTE_VALID;
+
+	    if(flags & VM_MEM_NOT_CACHEABLE) {
+		if(!(flags & VM_MEM_GUARDED))
+			template |= INTEL_PTE_PTA;
+		template |= INTEL_PTE_NCACHE;
+	    }
+
 	    if (pmap != kernel_pmap)
 		template |= INTEL_PTE_USER;
 	    if (prot & VM_PROT_WRITE)
@@ -1987,6 +2011,13 @@ RetryPvList:
 	 *	only the pfn changes.
 	 */
 	template = pa_to_pte(pa) | INTEL_PTE_VALID;
+
+	if(flags & VM_MEM_NOT_CACHEABLE) {
+		if(!(flags & VM_MEM_GUARDED))
+			template |= INTEL_PTE_PTA;
+		template |= INTEL_PTE_NCACHE;
+	}
+
 	if (pmap != kernel_pmap)
 		template |= INTEL_PTE_USER;
 	if (prot & VM_PROT_WRITE)
@@ -2026,6 +2057,7 @@ pmap_change_wiring(
 	register int		i;
 	spl_t			spl;
 
+#if 0
 	/*
 	 *	We must grab the pmap system lock because we may
 	 *	change a pte_page queue.
@@ -2058,6 +2090,11 @@ pmap_change_wiring(
 	}
 
 	PMAP_READ_UNLOCK(map, spl);
+
+#else
+	return;
+#endif
+
 }
 
 /*
@@ -2136,7 +2173,7 @@ pmap_expand(
 	 *	Map the page to its physical address so that it
 	 *	can be found later.
 	 */
-	pa = m->phys_addr;
+	pa = m->phys_page;
 	vm_object_lock(pmap_object);
 	vm_page_insert(m, pmap_object, pa);
 	vm_page_lock_queues();
@@ -2208,6 +2245,22 @@ pmap_copy(
 }
 #endif/* 	0 */
 
+/*
+ * pmap_sync_caches_phys(ppnum_t pa)
+ * 
+ * Invalidates all of the instruction cache on a physical page and
+ * pushes any dirty data from the data cache for the same physical page
+ */
+ 
+void pmap_sync_caches_phys(ppnum_t pa)
+{
+	if (!(cpuid_features() & CPUID_FEATURE_SS))
+	{
+		__asm__ volatile("wbinvd");	
+	}
+	return;
+}
+
 int	collect_ref;
 int	collect_unref;
 
@@ -2924,7 +2977,7 @@ pmap_movepage(unsigned long from, unsigned long to, vm_size_t size)
 		PMAP_READ_UNLOCK(kernel_pmap, spl);
 
 		pmap_enter(kernel_pmap, to, i386_trunc_page(*pte),
-			VM_PROT_READ|VM_PROT_WRITE, *pte & INTEL_PTE_WIRED);
+			VM_PROT_READ|VM_PROT_WRITE, 0, *pte & INTEL_PTE_WIRED);
 
 		pmap_remove(kernel_pmap, from, from+PAGE_SIZE);
 
@@ -2975,5 +3028,12 @@ kern_return_t bmapmapr(vm_offset_t va) {
 }
 #endif
 
-
-
+/* temporary workaround */
+boolean_t
+coredumpok(vm_map_t map, vm_offset_t va)
+{
+  pt_entry_t *ptep;
+  ptep = pmap_pte(map->pmap, va);
+  if (0 == ptep) return FALSE;
+  return ((*ptep & (INTEL_PTE_NCACHE|INTEL_PTE_WIRED)) != (INTEL_PTE_NCACHE|INTEL_PTE_WIRED));
+}
