@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -70,7 +73,6 @@
 #include <kern/task.h>
 #include <kern/counters.h>
 #include <kern/thread_act.h>
-#include <kern/thread_pool.h>
 #include <mach/mach_port_server.h>
 #include <vm/vm_map.h>
 #include <vm/vm_kern.h>
@@ -238,7 +240,7 @@ mach_port_names(
 		/* upper bound on number of names in the space */
 
 		bound = space->is_table_size + space->is_tree_total;
-		size_needed = round_page(bound * sizeof(mach_port_name_t));
+		size_needed = round_page_32(bound * sizeof(mach_port_name_t));
 
 		if (size_needed <= size)
 			break;
@@ -265,11 +267,20 @@ mach_port_names(
 
 		kr = vm_map_wire(ipc_kernel_map, addr1, addr1 + size,
 				     VM_PROT_READ|VM_PROT_WRITE, FALSE);
-		assert(kr == KERN_SUCCESS);
+		if (kr != KERN_SUCCESS) {
+			kmem_free(ipc_kernel_map, addr1, size);
+			kmem_free(ipc_kernel_map, addr2, size);
+			return KERN_RESOURCE_SHORTAGE;
+		}
 
 		kr = vm_map_wire(ipc_kernel_map, addr2, addr2 + size,
 				     VM_PROT_READ|VM_PROT_WRITE, FALSE);
-		assert(kr == KERN_SUCCESS);
+		if (kr != KERN_SUCCESS) {
+			kmem_free(ipc_kernel_map, addr1, size);
+			kmem_free(ipc_kernel_map, addr2, size);
+			return KERN_RESOURCE_SHORTAGE;
+		}
+
 	}
 	/* space is read-locked and active */
 
@@ -321,7 +332,7 @@ mach_port_names(
 		vm_size_t vm_size_used;
 
 		size_used = actual * sizeof(mach_port_name_t);
-		vm_size_used = round_page(size_used);
+		vm_size_used = round_page_32(size_used);
 
 		/*
 		 *	Make used memory pageable and get it into
@@ -499,7 +510,7 @@ mach_port_allocate_name(
 	if (!MACH_PORT_VALID(name))
 		return KERN_INVALID_VALUE;
 
-	kr = mach_port_allocate_full (space, right, SUBSYSTEM_NULL,
+	kr = mach_port_allocate_full (space, right, MACH_PORT_NULL,
 					&qos, &name);
 	return (kr);
 }
@@ -531,7 +542,7 @@ mach_port_allocate(
 	kern_return_t		kr;
 	mach_port_qos_t		qos = qos_template;
 
-	kr = mach_port_allocate_full (space, right, SUBSYSTEM_NULL,
+	kr = mach_port_allocate_full (space, right, MACH_PORT_NULL,
 					&qos, namep);
 	return (kr);
 }
@@ -564,50 +575,13 @@ mach_port_allocate_qos(
 {
 	kern_return_t		kr;
 
-	if (qosp->name == TRUE)
+	if (qosp->name)
 		return KERN_INVALID_ARGUMENT;
-	kr = mach_port_allocate_full (space, right, SUBSYSTEM_NULL,
+	kr = mach_port_allocate_full (space, right, MACH_PORT_NULL,
 					qosp, namep);
 	return (kr);
 }
 
-/*
- *	Routine:	mach_port_allocate_subsystem [kernel call]
- *	Purpose:
- *		Allocates a receive right in a space.  Like
- *		mach_port_allocate, except that the caller specifies an
- *		RPC subsystem that is to be used to implement RPC's to the
- *		port. When possible, allocate rpc subsystem ports without
- *		nms, since within RPC ports are intended to be used for
- *		identity only (i.e. nms is painful in the distributed case
- *		and we don't need or want it for RPC anyway).
- *	Conditions:
- *		Nothing locked.
- *	Returns:
- *		KERN_SUCCESS		The right is allocated.
- *		KERN_INVALID_TASK	The space is null.
- *		KERN_INVALID_TASK	The space is dead.
- *		KERN_RESOURCE_SHORTAGE	Couldn't allocate memory.
- *		KERN_NO_SPACE		No room in space for another right.
- *		KERN_INVALID_ARGUMENT	bogus subsystem
- */
-
-kern_return_t
-mach_port_allocate_subsystem(
-	ipc_space_t		space,
-	subsystem_t		subsystem,
-	mach_port_name_t	*namep)
-{
-	kern_return_t		kr;
-	ipc_port_t		port;
-	mach_port_qos_t		qos = qos_template;
-
-	kr = mach_port_allocate_full (space, 
-					MACH_PORT_RIGHT_RECEIVE,
-					subsystem, &qos, namep);
-	return (kr);
-}
-
 /*
  *	Routine:	mach_port_allocate_full [kernel call]
  *	Purpose:
@@ -631,7 +605,7 @@ kern_return_t
 mach_port_allocate_full(
 	ipc_space_t		space,
 	mach_port_right_t	right,
-	subsystem_t		subsystem,
+	mach_port_t		proto,
 	mach_port_qos_t		*qosp,
 	mach_port_name_t	*namep)
 {
@@ -641,6 +615,9 @@ mach_port_allocate_full(
 	if (space == IS_NULL)
 		return (KERN_INVALID_TASK);
 
+	if (proto != MACH_PORT_NULL)
+		return (KERN_INVALID_VALUE);
+
 	if (qosp->name) {
 		if (!MACH_PORT_VALID (*namep))
 			return (KERN_INVALID_VALUE);
@@ -648,11 +625,6 @@ mach_port_allocate_full(
 			return (KERN_FAILURE);
 	}
 
-	if (subsystem != SUBSYSTEM_NULL) {
-		if (right != MACH_PORT_RIGHT_RECEIVE)
-			return (KERN_INVALID_VALUE);
-	}
-
 	if (qosp->prealloc) {
 		mach_msg_size_t size = qosp->len + MAX_TRAILER_SIZE;
 		if (right != MACH_PORT_RIGHT_RECEIVE)
@@ -676,10 +648,6 @@ mach_port_allocate_full(
 			if (qosp->prealloc) 
 				ipc_kmsg_set_prealloc(kmsg, port);
 
-			if (subsystem != SUBSYSTEM_NULL) {
-				port->ip_subsystem = &subsystem->user;
-				subsystem_reference (subsystem);
-			}
 			ip_unlock(port);
 
 		} else if (qosp->prealloc)
@@ -1160,7 +1128,7 @@ mach_port_get_set_status(
 		/* didn't have enough memory; allocate more */
 
 		kmem_free(ipc_kernel_map, addr, size);
-		size = round_page(actual * sizeof(mach_port_name_t)) + PAGE_SIZE;
+		size = round_page_32(actual * sizeof(mach_port_name_t)) + PAGE_SIZE;
 	}
 
 	if (actual == 0) {
@@ -1172,7 +1140,7 @@ mach_port_get_set_status(
 		vm_size_t vm_size_used;
 
 		size_used = actual * sizeof(mach_port_name_t);
-		vm_size_used = round_page(size_used);
+		vm_size_used = round_page_32(size_used);
 
 		/*
 		 *	Make used memory pageable and get it into
@@ -1269,7 +1237,7 @@ mach_port_move_member(
 		assert(nset != IPS_NULL);
 	}
 	ip_lock(port);
-	ipc_pset_remove_all(port);
+	ipc_pset_remove_from_all(port);
 
 	if (nset != IPS_NULL) {
 		ips_lock(nset);
