@@ -3,25 +3,32 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
+
 #ifndef	_KERN_DEBUG_H_
 #define _KERN_DEBUG_H_
 
+#include <sys/appleapiopts.h>
+
+#ifdef	__APPLE_API_PRIVATE
 
 extern unsigned int	systemLogDiags;
 
@@ -38,7 +45,10 @@ extern unsigned int     current_debugger;
 
 extern unsigned int     active_debugger;
 extern unsigned int 	debug_mode; 
-extern unsigned int	disableDebugOuput;
+extern unsigned int		disableDebugOuput;
+
+extern unsigned int panicDebugging;
+extern unsigned int	logPanicDataToScreen;
 
 extern int db_run_mode;
 
@@ -57,6 +67,13 @@ extern char	*panicstr;
 
 extern unsigned int	nestedpanic;
 
+extern char *debug_buf;
+extern char *debug_buf_ptr;
+extern unsigned int debug_buf_size;
+
+extern void	debug_log_init(void);
+extern void	debug_putc(char);
+
 #endif /* MACH_KERNEL_PRIVATE */
 
 #define DB_HALT		0x1
@@ -65,5 +82,10 @@ extern unsigned int	nestedpanic;
 #define DB_KPRT		0x8
 #define DB_KDB		0x10
 #define DB_SLOG		0x20
+#define DB_ARP          0x40
+#define DB_KDP_BP_DIS   0x80
+#define DB_LOG_PI_SCRN	0x100
+
+#endif	/* __APPLE_API_PRIVATE */
 
 #endif	/* _KERN_DEBUG_H_ */
