@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -229,16 +232,9 @@ host_info(
 	}
 
 	/*
-	 * JMM - Temporary check to see if semaphore traps are
-	 * supported on this machine.  Sadly, just trying to call
-	 * the traps gets your process terminated instead of
-	 * returning an error, so we have to query during mach_init
-	 * to see if the machine supports them.
-	 *
-	 * KERN_INVALID_ARGUMENT - kernel has no semaphore traps
-	 * KERN_SUCCESS - kernel has sema traps (up to semaphore_signal_wait)
-	 * KERN_SEMAPHORE_DESTROYED - kernel has the latest semaphore traps
+	 * Gestalt for various trap facilities.
 	 */
+	case HOST_MACH_MSG_TRAP:
 	case HOST_SEMAPHORE_TRAPS:
 	{
 		*count = 0;
@@ -265,7 +261,7 @@ host_statistics(
 
 	case HOST_LOAD_INFO: {
 		register host_load_info_t load_info;
-		extern integer_t avenrun[3], mach_factor[3];
+		extern uint32_t avenrun[3], mach_factor[3];
 
 		if (*count < HOST_LOAD_INFO_COUNT)
 			return(KERN_FAILURE);
@@ -518,7 +514,7 @@ host_processor_info(
 		if (machine_slot[i].is_cpu)
 			num++;
 
-	size = (vm_size_t)round_page(num * count * sizeof(natural_t));
+	size = (vm_size_t)round_page_32(num * count * sizeof(natural_t));
 
 	kr = vm_allocate(ipc_kernel_map, &addr, size, TRUE);
 	if (kr != KERN_SUCCESS)
