@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -47,8 +50,6 @@
  * any improvements or extensions that they make and grant Carnegie Mellon
  * the rights to redistribute these changes.
  */
-/*
- */
 
 /*
  *	kern/host.h
@@ -61,12 +62,17 @@
 #define _KERN_HOST_H_
 
 #include <mach/mach_types.h>
-#include <mach/exception_types.h>
-#include <kern/kern_types.h>
+
+#include <sys/appleapiopts.h>
+
+#ifdef	__APPLE_API_PRIVATE
 
 #ifdef MACH_KERNEL_PRIVATE
 #include <kern/lock.h>
 #include <kern/exception.h>
+#include <mach/exception_types.h>
+#include <kern/kern_types.h>
+
 
 struct	host {
 	decl_mutex_data(,lock)		/* lock to protect exceptions */
@@ -86,6 +92,8 @@ extern host_data_t	realhost;
 
 #endif /* MACH_KERNEL_PRIVATE */
 
+#endif	/* __APPLE_API_PRIVATE */
+
 /*
  * Access routines for inside the kernel.
  */
