@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -56,6 +59,7 @@
  *	Task and thread related IPC functions.
  */
 
+#include <mach/mach_types.h>
 #include <mach/boolean.h>
 #include <mach_rt.h>
 #include <mach/kern_return.h>
@@ -116,8 +120,6 @@ ipc_task_init(
 		for (i = FIRST_EXCEPTION; i < EXC_TYPES_COUNT; i++) {
 			task->exc_actions[i].port = IP_NULL;
 		}/* for */
-		task->exc_actions[EXC_MACH_SYSCALL].port = 
-			ipc_port_make_send(realhost.host_self);
 		task->itk_host = ipc_port_make_send(realhost.host_self);
 		task->itk_bootstrap = IP_NULL;
 		for (i = 0; i < TASK_PORT_REGISTER_MAX; i++)
@@ -309,9 +311,6 @@ ipc_thr_act_init(task_t task, thread_act_t thr_act)
 	for (i = FIRST_EXCEPTION; i < EXC_TYPES_COUNT; i++)
 		thr_act->exc_actions[i].port = IP_NULL;
 
-	thr_act->exc_actions[EXC_MACH_SYSCALL].port =
-					ipc_port_make_send(realhost.host_self);
-
 	ipc_kobject_set(kport, (ipc_kobject_t) thr_act, IKOT_ACT);
 }
 
