@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -50,8 +53,8 @@
 #include <kern/zalloc.h>
 #include <kern/mach_param.h>
 
-unsigned int semaphore_event;
-#define SEMAPHORE_EVENT ((event_t)&semaphore_event)
+static unsigned int semaphore_event;
+#define SEMAPHORE_EVENT ((event64_t)&semaphore_event)
 
 zone_t semaphore_zone;
 unsigned int semaphore_max = SEMAPHORE_MAX;
@@ -188,7 +191,7 @@ semaphore_destroy(
 	semaphore->count = 0;
 
 	if (old_count < 0) {
-		wait_queue_wakeup_all_locked(&semaphore->wait_queue,
+		wait_queue_wakeup64_all_locked(&semaphore->wait_queue,
 					     SEMAPHORE_EVENT,
 					     THREAD_RESTART,
 					     TRUE);		/* unlock? */
@@ -235,7 +238,7 @@ semaphore_signal_internal(
 
 	if (thread_act != THR_ACT_NULL) {
 		if (semaphore->count < 0) {
-			kr = wait_queue_wakeup_thread_locked(
+			kr = wait_queue_wakeup64_thread_locked(
 			        	&semaphore->wait_queue,
 					SEMAPHORE_EVENT,
 					thread_act->thread,
@@ -254,7 +257,7 @@ semaphore_signal_internal(
 
 		if (old_count < 0) {
 			semaphore->count = 0;  /* always reset */
-			kr = wait_queue_wakeup_all_locked(
+			kr = wait_queue_wakeup64_all_locked(
 					&semaphore->wait_queue,
 					SEMAPHORE_EVENT,
 					THREAD_AWAKENED,
@@ -270,7 +273,7 @@ semaphore_signal_internal(
 	}
 	
 	if (semaphore->count < 0) {
-		if (wait_queue_wakeup_one_locked(
+		if (wait_queue_wakeup64_one_locked(
 					&semaphore->wait_queue,
 					SEMAPHORE_EVENT,
 					THREAD_AWAKENED,
@@ -582,10 +585,11 @@ semaphore_wait_internal(
 		kr = KERN_OPERATION_TIMED_OUT;
 	} else {		
 		wait_semaphore->count = -1;  /* we don't keep an actual count */
-		(void)wait_queue_assert_wait_locked(&wait_semaphore->wait_queue,
-					      SEMAPHORE_EVENT,
-					      THREAD_ABORTSAFE,
-					      FALSE); /* unlock? */
+		(void)wait_queue_assert_wait64_locked(
+					&wait_semaphore->wait_queue,
+					SEMAPHORE_EVENT,
+					THREAD_ABORTSAFE,
+					FALSE); /* unlock? */
 	}
 	semaphore_unlock(wait_semaphore);
 	splx(spl_level);
@@ -671,7 +675,7 @@ semaphore_wait_internal(
 		self->sth_signalsemaphore = signal_semaphore;
 		wait_result = thread_block(continuation);
 	} else {
-		wait_result = thread_block((void (*)(void))0);
+		wait_result = thread_block(THREAD_CONTINUE_NULL);
 	}
 
 	/*
