@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -82,7 +85,10 @@
 #include <mach/mach_types.h>
 #include <vm/pmap.h>
 
-#ifdef MACH_KERNEL_PRIVATE
+#ifdef	__APPLE_API_PRIVATE
+
+#ifdef	MACH_KERNEL_PRIVATE
+
 #include <mach/boolean.h>
 #include <mach/port.h>
 #include <mach/time_value.h>
@@ -142,12 +148,10 @@ typedef struct task {
 	/* User-visible scheduling information */
 	integer_t		user_stop_count;	/* outstanding stops */
 
-	integer_t		importance;		/* system-wide importance */
+	task_role_t		role;
 
-	integer_t		priority;		/* base priority */
-	integer_t		max_priority;	/* maximum priority */
-
-	integer_t		policy;			/* obsolete */
+	integer_t		priority;			/* base priority for threads */
+	integer_t		max_priority;		/* maximum priority for threads */
 
 	/* Task security token */
 	security_token_t sec_token;
@@ -174,10 +178,6 @@ typedef struct task {
 
 	struct ipc_space *itk_space;
 
-	/* RPC subsystem information */
-	queue_head_t	subsystem_list;	/* list of subsystems */
-	int		subsystem_count;/* number of subsystems */
-
 	/* Synchronizer ownership information */
 	queue_head_t	semaphore_list;		/* list of owned semaphores   */
 	queue_head_t	lock_set_list;		/* list of owned lock sets    */
@@ -211,9 +211,12 @@ typedef struct task {
 	void *bsd_info;
 #endif  
 	vm_offset_t	system_shared_region;
+	vm_offset_t	dynamic_working_set;
+	uint32_t taskFeatures[2];		/* Special feature for this task */
+#define tf64BitAddr	0x80000000		/* Task has 64-bit addressing */
+#define tf64BitData	0x40000000		/* Task has 64-bit data registers */
 } Task;
 
-
 #define task_lock(task)		mutex_lock(&(task)->lock)
 #define task_lock_try(task)	mutex_try(&(task)->lock)
 #define task_unlock(task)	mutex_unlock(&(task)->lock)
@@ -223,6 +226,8 @@ typedef struct task {
 #define	itk_lock(task)		mutex_lock(&(task)->itk_lock_data)
 #define	itk_unlock(task)	mutex_unlock(&(task)->itk_lock_data)
 
+#define task_reference_locked(task) ((task)->ref_count++)
+
 /*
  *	Internal only routines
  */
@@ -237,28 +242,14 @@ extern kern_return_t	task_create_local(
 				boolean_t	kernel_loaded,
 				task_t		*child_task);	/* OUT */
 
-#if	MACH_HOST
-/* Freeze and unfreeze task from being reassigned processor sets */
-extern void		task_freeze(
-				task_t		task);
-
-extern void		task_unfreeze(
-				task_t		task);
-#endif	/* MACH_HOST */
-
 extern void		consider_task_collect(void);
 
 #define	current_task_fast()	(current_act_fast()->task)
 #define current_task()		current_task_fast()
 
-#else /* !MACH_KERNEL_PRIVATE */
-
-extern task_t current_task(void);
-
-#endif /* !MACH_KERNEL_PRIVATE */
+#endif	/* MACH_KERNEL_PRIVATE */
 
 extern task_t		kernel_task;
-extern boolean_t	is_kerneltask(task_t);
 
 /* Temporarily hold all threads in a task */
 extern kern_return_t	task_hold(
@@ -272,14 +263,11 @@ extern kern_return_t	task_release(
 extern kern_return_t	task_halt(
 				task_t	task);
 
-/* Take reference on task (make sure it doesn't go away) */
-extern void		task_reference(task_t	task);
-
-/* Try to take a reference on task, return false if it would block */
-extern boolean_t	task_reference_try(task_t task);
-
-/* Remove reference to task */
-extern void		task_deallocate(task_t	task);
+#if defined(MACH_KERNEL_PRIVATE) || defined(BSD_BUILD)
+extern kern_return_t	task_importance(
+							task_t			task,
+							integer_t		importance);
+#endif
 
 /* JMM - should just be temporary (implementation in bsd_kern still) */
 extern void 	*get_bsdtask_info(task_t);
@@ -287,4 +275,21 @@ extern void	set_bsdtask_info(task_t,void *);
 extern vm_map_t get_task_map(task_t);
 extern vm_map_t	swap_task_map(task_t, vm_map_t);
 extern pmap_t	get_task_pmap(task_t);
+
+extern boolean_t	task_reference_try(task_t task);
+
+#endif	/* __APPLE_API_PRIVATE */
+
+#if		!defined(MACH_KERNEL_PRIVATE)
+
+extern task_t	current_task(void);
+
+#endif	/* MACH_KERNEL_TASK */
+
+/* Take reference on task (make sure it doesn't go away) */
+extern void		task_reference(task_t	task);
+
+/* Remove reference to task */
+extern void		task_deallocate(task_t	task);
+
 #endif	/* _KERN_TASK_H_ */
