@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -67,6 +70,11 @@ mutex_t		task_swapper_lock;	/* protects above queue */
 
 #define task_swapper_lock()	mutex_lock(&task_swapper_lock)
 #define task_swapper_unlock()	mutex_unlock(&task_swapper_lock)
+#define task_swapper_wakeup()	thread_wakeup((event_t)&swapout_thread_q)
+#define task_swapper_sleep()	thread_sleep_mutex((event_t)&swapout_thread_q, \
+							&task_swapper_lock,    \
+						   	THREAD_UNINT)
+
 
 queue_head_t	eligible_tasks;		/* tasks eligible for swapout */
 mutex_t		task_swapout_list_lock;	/* protects above queue */
@@ -450,7 +458,7 @@ thread_swapout_enqueue(thread_act_t thr_act)
 	} else {
 		queue_enter(&swapout_thread_q, thr_act,
 			    thread_act_t, swap_queue);
-		thread_wakeup((event_t)&swapout_thread_q);
+		task_swapper_wakeup();
 	}
 	act_unlock(thr_act);
 	task_swapper_unlock();
@@ -475,8 +483,8 @@ task_swap_swapout_thread(void)
 
 	spllo();
 
+	task_swapper_lock();
 	while (TRUE) {
-		task_swapper_lock();
 		while (! queue_empty(&swapout_thread_q)) {
 
 			queue_remove_first(&swapout_thread_q, thr_act,
@@ -584,9 +592,7 @@ task_swap_swapout_thread(void)
 			act_deallocate(thr_act);
 			task_swapper_lock();
 		}
-		assert_wait((event_t)&swapout_thread_q, THREAD_UNINT);
-		task_swapper_unlock();
-		thread_block((void (*)(void)) 0);
+		task_swapper_sleep();
 	}
 }
 
@@ -685,7 +691,7 @@ task_swapin(task_t task, boolean_t make_unswappable)
 			task->swap_flags |= TASK_SW_WANT_IN;
 			assert_wait((event_t)&task->swap_state, THREAD_UNINT);
 			task_unlock(task);
-			thread_block((void (*)(void)) 0);
+			thread_block(THREAD_CONTINUE_NULL);
 			TASK_STATS_INCR(task_sw_race_coming_in);
 			return(KERN_SUCCESS);
 	    case TASK_SW_UNSWAPPABLE:
@@ -1213,7 +1219,7 @@ task_swapper(void)
 		}
 		if (task_swap_debug)
 			printf("task_swapper: blocking\n");
-		thread_block((void (*)(void)) 0);
+		thread_block(THREAD_CONTINUE_NULL);
 		if (timeout) {
 			thread_cancel_timeout(current_thread());
 		}
