@@ -4,19 +4,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -35,6 +38,8 @@
 #ifndef _KERN_THREAD_CALL_H_
 #define _KERN_THREAD_CALL_H_
 
+#include <sys/appleapiopts.h>
+
 #include <mach/mach_types.h>
 
 #include <kern/clock.h>
@@ -85,11 +90,16 @@ thread_call_free(
 	thread_call_t		call
 );
 
+#ifdef	__APPLE_API_PRIVATE
+
+#ifdef	__APPLE_API_OBSOLETE
+
 /*
  * This portion of the interface
  * is OBSOLETE and DEPRECATED.  It
  * will disappear shortly.
  */
+
 void
 thread_call_func(
 	thread_call_func_t		func,
@@ -109,8 +119,11 @@ thread_call_func_cancel(
 	thread_call_param_t		param,
 	boolean_t				cancel_all
 );
+
 /* End OBSOLETE and DEPRECATED */
 
+#endif	/* __APPLE_API_OBSOLETE */
+
 #ifdef MACH_KERNEL_PRIVATE
 #include <kern/call_entry.h>
 
@@ -126,8 +139,14 @@ thread_call_setup(
 	thread_call_param_t		param0
 );
 
+void
+call_thread_block(void),
+call_thread_unblock(void);
+
 #endif /* MACH_KERNEL_PRIVATE */
 
+#endif	/* __APPLE_API_PRIVATE */
+
 #if		!defined(MACH_KERNEL_PRIVATE) && !defined(ABSOLUTETIME_SCALAR_TYPE)
 
 #include <libkern/OSBase.h>
