@@ -4,19 +4,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -32,6 +35,7 @@
 #include <mach/mach_types.h>
 
 #include <kern/clock.h>
+#include <kern/processor.h>
 
 #include <kern/timer_call.h>
 #include <kern/call_entry.h>
@@ -43,8 +47,6 @@ queue_head_t
 	delayed_call_queues[NCPUS];
 
 static struct {
-	int		pending_num,
-			pending_hiwat;
 	int		delayed_num,
 			delayed_hiwat;
 } timer_calls;
@@ -131,30 +133,6 @@ _delayed_call_dequeue(
 	call->state = IDLE;
 }
 
-static __inline__
-void
-_pending_call_enqueue(
-	queue_t					queue,
-	timer_call_t			call)
-{
-	enqueue_tail(queue, qe(call));
-	if (++timer_calls.pending_num > timer_calls.pending_hiwat)
-		timer_calls.pending_hiwat = timer_calls.pending_num;
-
-	call->state = PENDING;
-}
-
-static __inline__
-void
-_pending_call_dequeue(
-	timer_call_t			call)
-{
-	(void)remque(qe(call));
-	timer_calls.pending_num--;
-
-	call->state = IDLE;
-}
-
 static __inline__
 void
 _set_delayed_call_timer(
@@ -175,11 +153,9 @@ timer_call_enter(
 	s = splclock();
 	simple_lock(&timer_call_lock);
 
-	if (call->state == PENDING)
-		_pending_call_dequeue(call);
-	else if (call->state == DELAYED)
+	if (call->state == DELAYED)
 		_delayed_call_dequeue(call);
-	else if (call->state == IDLE)
+	else
 		result = FALSE;
 
 	call->param1	= 0;
@@ -211,11 +187,9 @@ timer_call_enter1(
 	s = splclock();
 	simple_lock(&timer_call_lock);
 
-	if (call->state == PENDING)
-		_pending_call_dequeue(call);
-	else if (call->state == DELAYED)
+	if (call->state == DELAYED)
 		_delayed_call_dequeue(call);
-	else if (call->state == IDLE)
+	else
 		result = FALSE;
 
 	call->param1	= param1;
@@ -244,9 +218,7 @@ timer_call_cancel(
 	s = splclock();
 	simple_lock(&timer_call_lock);
 
-	if (call->state == PENDING)
-		_pending_call_dequeue(call);
-	else if (call->state == DELAYED)
+	if (call->state == DELAYED)
 		_delayed_call_dequeue(call);
 	else
 		result = FALSE;
@@ -280,6 +252,42 @@ timer_call_is_delayed(
 	return (result);
 }
 
+/*
+ * Called at splclock.
+ */
+
+void
+timer_call_shutdown(
+	processor_t			processor)
+{
+	timer_call_t		call;
+	queue_t				delayed, delayed1;
+
+	assert(processor != current_processor());
+
+	delayed = &delayed_call_queues[processor->slot_num];
+	delayed1 = &delayed_call_queues[cpu_number()];
+
+	simple_lock(&timer_call_lock);
+
+	call = TC(queue_first(delayed));
+
+	while (!queue_end(delayed, qe(call))) {
+		_delayed_call_dequeue(call);
+
+		_delayed_call_enqueue(delayed1, call);
+
+		call = TC(queue_first(delayed));
+	}
+
+	call = TC(queue_first(delayed1));
+
+	if (!queue_end(delayed1, qe(call)))
+		_set_delayed_call_timer(call);
+
+	simple_unlock(&timer_call_lock);
+}
+
 static
 void
 timer_call_interrupt(
