@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -50,7 +53,16 @@ int diagCall(struct savearea *save);
 #define dgBMphys 5
 #define dgUnMap 6
 #define dgBootScreen 7
-#define dgflush 8
+#define dgFlush 8
+#define dgAlign 9
+#define dgprw 10
+#define dgmck 11
+#define dg64 12
+#define dgProbeRead 13
+#define dgCPNull 14
+#define dgPerfMon 15
+#define dgMapPage 16
+#define dgScom 17
 
 
 typedef struct diagWork {			/* Diagnostic work area */
@@ -67,6 +79,12 @@ typedef struct diagWork {			/* Diagnostic work area */
 #define enaDiagSCsb  28
 #define enaDiagDM  0x00000010
 #define enaDiagSDMb  27
+#define enaDiagEM  0x00000020
+#define enaDiagEMb  26
+#define enaDiagTrap  0x00000040
+#define enaDiagTrapb  25
+#define enaNotifyEM  0x00000080
+#define enaNotifyEMb  24
 /* Suppress lock checks */
 #define disLkType 0x80000000
 #define disLktypeb 0
@@ -86,7 +104,16 @@ typedef struct diagWork {			/* Diagnostic work area */
 
 } diagWork;
 
+typedef struct scomcomm {
+	uint16_t	scomcpu;	/* CPU number */
+	uint16_t	scomfunc;	/* 0 = read; 1 = write */
+	uint32_t	scomreg;	/* SCOM register */
+	uint64_t	scomstat;	/* returned status */
+	uint64_t	scomdata;	/* input for write,  output for read */
+} scomcomm;
+
 extern diagWork dgWork;
+extern int diagTrap(struct savearea *, unsigned int);
 
 
 #endif /* _DIAGNOSTICS_H_ */
