@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -41,6 +44,7 @@
 #include <kern/spl.h>
 
 #include <machine/mach_param.h>	/* HZ */
+#include <machine/commpage.h>
 #include <ppc/proc_reg.h>
 
 #include <pexpert/pexpert.h>
@@ -106,7 +110,7 @@ static struct rtclock {
 
 	clock_timer_func_t	timer_expire;
 
-	timer_call_data_t	alarm[NCPUS];
+	timer_call_data_t	alarm_timer;
 
 	/* debugging */
 	uint64_t			last_abstime[NCPUS];
@@ -120,6 +124,9 @@ static boolean_t		rtclock_initialized;
 static uint64_t			rtclock_tick_deadline[NCPUS];
 static uint64_t		 	rtclock_tick_interval;
 
+static uint32_t			rtclock_sec_divisor;
+static uint32_t			rtclock_ns_per_tick;
+
 static void		timespec_to_absolutetime(
 							mach_timespec_t		timespec,
 							uint64_t			*result);
@@ -181,6 +188,9 @@ timebase_callback(
 	LOCK_RTC(s);
 	rtclock.timebase_const.numer = numer;
 	rtclock.timebase_const.denom = denom;
+    rtclock_sec_divisor = freq->timebase_num / freq->timebase_den;
+    rtclock_ns_per_tick = NSEC_PER_SEC / rtclock_sec_divisor;
+    commpage_set_timestamp(0,0,0,0);
 	UNLOCK_RTC(s);
 }
 
@@ -190,13 +200,10 @@ timebase_callback(
 int
 sysclk_config(void)
 {
-	int			i;
-
 	if (cpu_number() != master_cpu)
 		return(1);
 
-	for (i = 0; i < NCPUS; i++)
-		timer_call_setup(&rtclock.alarm[i], rtclock_alarm_timer, NULL);
+	timer_call_setup(&rtclock.alarm_timer, rtclock_alarm_timer, NULL);
 
 	simple_lock_init(&rtclock.lock, ETAP_MISC_RT_CLOCK);
 
@@ -537,7 +544,7 @@ sysclk_setalarm(
 	uint64_t			abstime;
 
 	timespec_to_absolutetime(*deadline, &abstime);
-	timer_call_enter(&rtclock.alarm[cpu_number()], abstime);
+	timer_call_enter(&rtclock.alarm_timer, abstime);
 }
 
 /*
@@ -561,6 +568,62 @@ calend_init(void)
 	return (1);
 }
 
+/*
+ * Get the current clock microtime and sync the timestamp
+ * on the commpage.  Only called from ppc_gettimeofday(),
+ * ie in response to a system call from user mode.
+ */
+void
+clock_gettimeofday(
+	uint32_t			*secp,
+	uint32_t			*usecp)
+{
+	uint64_t			now;
+    UnsignedWide		wide_now;
+	UnsignedWide		t64;
+	uint32_t			t32;
+	uint32_t			numer, denom;
+    uint32_t			secs,usecs;
+    mach_timespec_t		curr_time;
+	spl_t				s;
+
+	LOCK_RTC(s);
+	if (!rtclock.calend_is_set) {
+		UNLOCK_RTC(s);
+		return;
+	}
+
+	numer = rtclock.timebase_const.numer;
+	denom = rtclock.timebase_const.denom;
+
+	clock_get_uptime(&now);
+    wide_now = *((UnsignedWide*) &now);
+
+	umul_64by32(wide_now, numer, &t64, &t32);
+
+	udiv_96by32(t64, t32, denom, &t64, &t32);
+
+	udiv_96by32to32and32(t64, t32, NSEC_PER_SEC,
+								&curr_time.tv_sec, &curr_time.tv_nsec);
+
+	ADD_MACH_TIMESPEC(&curr_time, &rtclock.calend_offset);
+    
+	secs = curr_time.tv_sec;
+	usecs = curr_time.tv_nsec / NSEC_PER_USEC;
+    *secp = secs;
+    *usecp = usecs;
+
+    t32 = curr_time.tv_nsec - (usecs * NSEC_PER_USEC);
+    t32 = t32 / rtclock_ns_per_tick;
+    now -= t32;
+
+    commpage_set_timestamp(now,secs,usecs,rtclock_sec_divisor);
+    
+	UNLOCK_RTC(s);
+
+	return;
+} 
+
 /*
  * Get the current clock time.
  */
@@ -598,6 +661,7 @@ calend_settime(
 	rtclock.calend_offset = *new_time;
 	SUB_MACH_TIMESPEC(&rtclock.calend_offset, &curr_time);
 	rtclock.calend_is_set = TRUE;
+    commpage_set_timestamp(0,0,0,0);  /* disable timestamp */
 	UNLOCK_RTC(s);
 
 	PESetGMTTimeOfDay(new_time->tv_sec);
@@ -645,8 +709,10 @@ clock_adjust_calendar(
 	spl_t		s;
 
 	LOCK_RTC(s);
-	if (rtclock.calend_is_set)
+	if (rtclock.calend_is_set) {
 		ADD_MACH_TIMESPEC_NSEC(&rtclock.calend_offset, nsec);
+        commpage_set_timestamp(0,0,0,0);  /* disable timestamp */
+    }
 	UNLOCK_RTC(s);
 }
 
@@ -666,6 +732,7 @@ clock_initialize_calendar(void)
 	rtclock.calend_offset.tv_nsec = 0;
 	SUB_MACH_TIMESPEC(&rtclock.calend_offset, &curr_time);
 	rtclock.calend_is_set = TRUE;
+    commpage_set_timestamp(0,0,0,0);  /* disable timestamp */
 	UNLOCK_RTC(s);
 }
 
@@ -753,7 +820,7 @@ rtclock_reset(void)
 void
 rtclock_intr(
 	int						device,
-	struct ppc_saved_state	*ssp,
+	struct savearea			*ssp,
 	spl_t					old_spl)
 {
 	uint64_t				abstime;
@@ -775,7 +842,7 @@ rtclock_intr(
 	if (	rtclock_tick_deadline[mycpu] <= abstime		) {
 		clock_deadline_for_periodic_event(rtclock_tick_interval, abstime,
 										  		&rtclock_tick_deadline[mycpu]);
-		hertz_tick(USER_MODE(ssp->srr1), ssp->srr0);
+		hertz_tick(USER_MODE(ssp->save_srr1), ssp->save_srr0);
 	}
 
 	clock_get_uptime(&abstime);
