@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -31,6 +34,7 @@
 #include <pexpert/device_tree.h>
 #include <pexpert/pe_images.h>
 #include <kern/debug.h>
+#include <kern/sched_prim.h>
 
 /* extern references */
 void pe_identify_machine(void);
@@ -113,55 +117,35 @@ int PE_initialize_console( PE_Video * info, int op )
     return 0;
 }
 
-static boolean_t find_image( const char * name,
-				void ** desc,
-				unsigned char ** data,
-				unsigned char ** clut )
-{
-    boolean_t	ok;
-#if 0
-    DTEntry	entry;
-    int		size;
-
-    // This is a little flawed now the device tree data
-    // is freed.
-    if( (kSuccess == DTLookupEntry(0, "/AAPL,images", &entry))
-     && (kSuccess == DTLookupEntry(entry, name, &entry)) ) {
-
-	ok = ( (kSuccess == DTGetProperty(entry, "desc",
-				desc, &size))
-            && (kSuccess == DTGetProperty(entry, "data",
-				(void **)data, &size)));
-
-        if( clut && (kSuccess != DTGetProperty(entry, "clut",
-				(void **)clut, &size)))
-	    *clut = appleClut8;
-    } else
-#endif
-	ok = FALSE;
-
-    return( ok );
-}
-
 void PE_init_iokit(void)
 {
     kern_return_t	ret;
-    void * 		desc;
-    unsigned char *	data;
-    unsigned char *	clut;
+    DTEntry		entry;
+    int			size;
+    int			i;
+    void **		map;
 
     PE_init_kprintf(TRUE);
     PE_init_printf(TRUE);
 
-    // init this now to get mace debugger for iokit startup
-    PE_init_ethernet_debugger();
+    if( kSuccess == DTLookupEntry(0, "/chosen/memory-map", &entry)) {
 
-    if( !find_image( "progress", &desc, &data, &clut)) {
-        clut = appleClut8;
-        desc = &default_progress;
-        data = default_progress_data;
+	boot_progress_element * bootPict;
+
+	if( kSuccess == DTGetProperty(entry, "BootCLUT", (void **) &map, &size))
+	    bcopy( map[0], appleClut8, sizeof(appleClut8) );
+
+	if( kSuccess == DTGetProperty(entry, "Pict-FailedBoot", (void **) &map, &size)) {
+
+	    bootPict = (boot_progress_element *) map[0];
+	    default_noroot.width  = bootPict->width;
+	    default_noroot.height = bootPict->height;
+	    default_noroot.dx     = 0;
+	    default_noroot.dy     = bootPict->yOffset;
+	    default_noroot_data   = &bootPict->data[0];
+	}
     }
-    vc_progress_initialize( desc, data, clut );
+    vc_progress_initialize( &default_progress, default_progress_data, (unsigned char *) appleClut8 );
 
     PE_initialize_console( (PE_Video *) 0, kPEAcquireScreen );
 
@@ -216,20 +200,15 @@ void PE_create_console( void )
 int PE_current_console( PE_Video * info )
 {
     *info = PE_state.video;
+    info->v_baseAddr = 0;
     return( 0);
 }
 
 void PE_display_icon(	unsigned int flags,
 			const char * name )
 {
-    void * 		desc;
-    unsigned char *	data;
-
-    if( !find_image( name, &desc, &data, 0)) {
-        desc = &default_roroot;
-        data = default_noroot_data;
-    }
-    vc_display_icon( desc, data );
+    if( default_noroot_data)
+	vc_display_icon( &default_noroot, default_noroot_data );
 }
 
 extern boolean_t PE_get_hotkey(
@@ -266,8 +245,8 @@ void PE_call_timebase_callback(void)
   struct timebase_freq_t timebase_freq;
   unsigned long          num, den, cnt;
   
-  num = gPEClockFrequencyInfo.bus_clock_rate_num * gPEClockFrequencyInfo.bus_to_dec_rate_num;
-  den = gPEClockFrequencyInfo.bus_clock_rate_den * gPEClockFrequencyInfo.bus_to_dec_rate_den;
+  num = gPEClockFrequencyInfo.timebase_frequency_num;
+  den = gPEClockFrequencyInfo.timebase_frequency_den;
   
   cnt = 2;
   while (cnt <= den) {
